/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/glint.h>

#include <boost/thread.hpp>
#include <GL/glx.h>

#include <iostream>
#include <string>
#include <cassert>

using namespace std;
using namespace glint;

class Drawer
{
public:
    Drawer(int xpos, int ypos)
        : xpos_(xpos), ypos_(ypos)
    {
    }

    void operator()();

private:
    int xpos_;
    int ypos_;
};

void
Drawer::
operator()()
{
    Runtime::Ptr rt = Runtime::create();
    GraphicsDisplay* d = rt->create_display(200, 200, xpos_, ypos_);
    d->set_visibility(GraphicsDisplay::Vis::visible);

#if 0
    while (true)
    {
        glShadeModel(GL_SMOOTH);
        glClearDepth(1.0f);
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LEQUAL);
        glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();

        GLdouble fW, fH, aspect;
        aspect = d->width() / d->height();
        fH = tan(45.0 / 360 * 3.1415926) * 0.1f;
        fW = fH * aspect;
        glFrustum(-fW, fW, -fH, fH, 0.1f, 255.0f);

        glMatrixMode(GL_MODELVIEW);
        glFlush();

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        glLoadIdentity();

        glTranslatef(-1.5f, 0.0f, -6.0f);

        glBegin(GL_TRIANGLES);
            glVertex3f( 0.0f,  1.0f,  0.0f);
            glVertex3f(-1.0f, -1.0f,  0.0f);
            glVertex3f( 1.0f, -1.0f,  0.0f);
        glEnd();

        glTranslatef(3.0f, 0.0f, 0.0f);

        glBegin(GL_QUADS);
            glVertex3f(-1.0f, 1.0f, 0.0f);
            glVertex3f( 1.0f, 1.0f, 0.0f);
            glVertex3f( 1.0f,-1.0f, 0.0f);
            glVertex3f(-1.0f,-1.0f, 0.0f);
        glEnd();
        d->swap_buffers();

        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        d->swap_buffers();
    }
#endif
}

int
main()
{
    for (int i = 0; i < 4; ++i)
    {
        for (int j = 0; j < 5; ++j)
        {
            boost::thread t(Drawer(j*200, i*200));
            sleep(1);
        }
    }
    sleep(30);
}
