/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_COLORCOMMON_H
#define GLINT_COLORCOMMON_H

#include <glint/GlintExceptions.h>

#include <cmath>
#include <string>
#include <sstream>

namespace glint
{

namespace   // Utility definitions and functions
{

constexpr const char* int_range = "0-255";
constexpr const char* float_range = "0.0-1.0";
constexpr const char* degree_range = "0-359";

inline
bool
value_in_range(int value)
{
    return value >= 0 && value <= 0xff;
}

inline
bool
value_in_range(float value)
{
    return value >= 0.0f && value <= 1.0f;
}

inline
float
to_float(int value)
{
    return value / 255.0f;
}

inline
int
to_int(float value)
{
    // Round to closest integer
    return floor((value + 1 / 255.0 / 2) * 255);
}

template<typename V>
std::string
value_msg(V val, const char* name)
{
    return value_in_range(val) ? std::string() : std::to_string(val) + " (" + name + ")";
}

template<typename T>
void
throw_if_not_in_range(T val, const char* name, const char* range)
{
    if (!(value_in_range(val)))
    {
        std::ostringstream msg;
        msg << "Invalid color argument outside [" << range << "] range: " << val << " (" << name << ")";
        throw InvalidArgumentException(msg.str());
    }
}

} // namespace

} // namespace glint

#endif
