/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_GLINTEXCEPTIONS_H
#define GLINT_GLINTEXCEPTIONS_H

#include <glint/Exception.h>

#include <exception>
#include <string>
#include <memory>

namespace glint
{

//
// Exception for OpenGL errors.
//

namespace internal
{
class OpenGLExceptionImpl;
}

class OpenGLException : public Exception
{
public:
    OpenGLException(const std::string& reason, int gl_error);
    OpenGLException(const OpenGLException&);
    OpenGLException& operator=(const OpenGLException&);
    virtual ~OpenGLException() noexcept;

    virtual const char* what() const noexcept;
    virtual std::exception_ptr self() const;

    int gl_error() const noexcept;
};

//
// Exception for invalid or out-of-range arguments.
//

namespace internal
{
class InvalidArgumentExceptionImpl;
}

class InvalidArgumentException : public Exception
{
public:
    explicit InvalidArgumentException(const std::string& reason);
    InvalidArgumentException(const InvalidArgumentException&);
    InvalidArgumentException& operator=(const InvalidArgumentException&);
    virtual ~InvalidArgumentException() noexcept;

    virtual const char* what() const noexcept;
    virtual std::exception_ptr self() const;
};

//
// Exception for logic errors (driving the API incorrectly).
//

namespace internal
{
class LogicExceptionImpl;
}

class LogicException : public Exception
{
public:
    explicit LogicException(const std::string& reason);
    LogicException(const LogicException&);
    LogicException& operator=(const LogicException&);
    virtual ~LogicException() noexcept;

    virtual const char* what() const noexcept;
    virtual std::exception_ptr self() const;
};

//
// Exception for error during shutdown.
//

namespace internal
{
class ShutdownExceptionImpl;
}

class ShutdownException : public Exception
{
public:
    explicit ShutdownException(const std::string& reason);
    ShutdownException(const ShutdownException&);
    ShutdownException& operator=(const ShutdownException&);
    virtual ~ShutdownException() noexcept;

    virtual const char* what() const noexcept;
    virtual std::exception_ptr self() const;
};

//
// Exception for file I/O errors.
//

namespace internal
{
class FileExceptionImpl;
}

class FileException : public Exception
{
public:
    FileException(const std::string& reason, int err);
    FileException(const FileException&);
    FileException& operator=(const FileException&);
    virtual ~FileException() noexcept;

    virtual const char* what() const noexcept;
    virtual std::exception_ptr self() const;

    int error() const noexcept;
};

} // namespace glint

#endif
