/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_GRAPHICSDISPLAY_H
#define GLINT_GRAPHICSDISPLAY_H

#include <glint/AbstractWindow.h>

#include <boost/utility.hpp>
#include <memory>

namespace glint
{

//
// Abstract base class for implementation by a concrete GraphicsDisplay.
//

class GraphicsDisplay : private boost::noncopyable
{
public:
    virtual int gl_major_version() const noexcept = 0;  // OpenGL major version
    virtual int gl_minor_version() const noexcept = 0;  // OpenGL minor version

    virtual int width() const noexcept = 0;       // Actual height as opposed to requested height
    virtual int height() const noexcept = 0;      // Actual width as opposed to requested width
    virtual int xpos() const noexcept = 0;        // Actual x position as opposed to requested position
    virtual int ypos() const noexcept = 0;        // Actual y position as opposed to requested position

    virtual void make_current() = 0;              // Assign the drawing context to the calling thread
    virtual void release_current() = 0;           // Release the drawing context
    virtual void swap_buffers() = 0;              // Swap the rendering buffers

    enum class Vis { invisible, visible };
    virtual Vis set_visibility(Vis v) = 0;        // Show or hide the window
    virtual Vis visibility() = 0;                 // Current visibility state

    enum class StackingOrder { bottom, top };
    virtual void set_stacking_order(StackingOrder order) = 0;  // Raise or lower the window

    virtual AbstractWindow* window() const noexcept = 0;  // Returns the associated window

protected:
    GraphicsDisplay();
    virtual ~GraphicsDisplay() noexcept;
};

} // namespace glint

#endif
