/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_OPENGL_H
#define GLINT_OPENGL_H

namespace glint
{

enum class GLEnv { OpenGL, OpenGLES };

} // namespace glint

#if defined(GLINT_GL) && defined(GLINT_GLES)

    #error "Exactly one (not both) of GLINT_GL (for OpenGL) or GLINT_GLES (for OpenGL ES)"
           " must be defined before including a glint header"

#elif defined(GLINT_GL)

    #include <GL/glew.h>

    namespace glint
    {

    const GLEnv gl_environment = GLEnv::OpenGL;

    } // namespace glint

#elif defined(GLINT_GLES)

    #include <GLES2/gl2.h>

    namespace glint
    {

    const GLEnv gl_environment = GLEnv::OpenGLES;

    } // namespace glint

#else

    #error "Either GLINT_GL (for OpenGL) or GLINT_GLES (for OpenGL ES) must be defined before including a glint header"

#endif

#endif
