/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_PROGRAM_H
#define GLINT_PROGRAM_H

#include <glint/util/DefinesPtrs.h>
#include <glint/Shader.h>
#include <glint/UniformVariable.h>
#include <glint/GLUtil.h>

#include <boost/utility.hpp>

#include <map>

namespace glint
{

namespace internal
{

class ProgramImpl;

} // namespace internal

class VertexShader;
class FragmentShader;

class Program : public util::DefinesPtrs<Program>, private boost::noncopyable
{
public:
    static UPtr create(const VertexShader& vs, const FragmentShader& fs);

    GLuint id() const noexcept;
    void use() const;
    bool in_use() const noexcept;

    GLuint vertex_shader_id() const;
    GLuint fragment_shader_id() const;

    typedef std::map<std::string, std::unique_ptr<UniformVariable>> UniformMap;

    UniformVariable& uniform(const std::string& name) const;
    const UniformMap& uniforms() const;

private:
    Program(const VertexShader& vs, const FragmentShader& fs);  // Class is final, instantiation only via create()
    ~Program() noexcept;

    std::shared_ptr<internal::ProgramImpl> p_;
    friend class std::default_delete<Program>;     // Because destructor is private
};

} // namespace glint

#endif
