/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_SHADER_H
#define GLINT_SHADER_H

#include <glint/util/DefinesPtrs.h>

#include <glint/OpenGL.h>
#include <boost/utility.hpp>

#include <string>
#include <vector>

namespace glint
{

class Shader : private boost::noncopyable       // TODO: should be a method on Program? or in glint namespace?
                                                // or delete this altogether?
{
public:
    static bool supports_compiled_shaders();

private:
    Shader();       // Not a class, really, just a class used as a namespace
    ~Shader();
};

namespace internal
{

class VertexShaderImpl;

} // namespace internal

class VertexShader : public util::DefinesPtrs<VertexShader>, private boost::noncopyable
{
public:
    static UPtr create(const std::string& source);
    static UPtr create_from_file(const std::string& filename);

    GLuint id() const noexcept;

private:
    explicit VertexShader(const std::string& source);   // Class is final, instnatiation only via create()
    ~VertexShader() noexcept;

    std::shared_ptr<internal::VertexShaderImpl> p_;
    friend class std::default_delete<VertexShader>;     // Because destructor is private
};

namespace internal
{

class FragmentShaderImpl;

} // namespace internal

class FragmentShader : public util::DefinesPtrs<FragmentShader>, private boost::noncopyable
{
public:
    static UPtr create(const std::string& source);
    static UPtr create_from_file(const std::string& filename);

    GLuint id() const noexcept;

private:
    explicit FragmentShader(const std::string& source); // Class is final, instantiation only via create()
    ~FragmentShader() noexcept;

    std::shared_ptr<internal::FragmentShaderImpl> p_;
    friend class std::default_delete<FragmentShader>;   // Because destructor is private
};

} // namespace glint

#endif
