/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_SHADERVARIABLE_H
#define GLINT_SHADERVARIABLE_H

#include <glint/OpenGL.h>
#include <boost/utility.hpp>

#include <memory>
#include <string>

namespace glint
{

namespace internal
{

class ShaderVariableImpl;

} // namespace internal

class ShaderVariable : boost::noncopyable
{
public:
    GLuint program_id() const noexcept;
    GLint location() const noexcept;
    std::string name() const noexcept;
    GLenum type() const noexcept;
    GLint size() const noexcept;

    virtual ~ShaderVariable();

protected:
    ShaderVariable(const std::shared_ptr<internal::ShaderVariableImpl>& derived);

    std::shared_ptr<internal::ShaderVariableImpl> pimpl() const noexcept;     // No need to reimplement in derived

private:
    std::shared_ptr<internal::ShaderVariableImpl> p_;
};

} // namespace glint

#endif
