/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_GENV_EGL_INTERNAL_EGLGENV_H
#define GLINT_GENV_EGL_INTERNAL_EGLGENV_H

#include <glint/genv/internal/AbstractGEnv.h>
#include <glint/genv/internal/AbstractWEnv.h>
#include <glint/genv/NativeDefs.h>
#include <glint/genv/egl/internal/EGLGraphicsDisplay.h>

namespace glint
{

namespace genv
{

namespace egl
{

namespace internal
{

class EGLGEnv : public glint::genv::internal::AbstractGEnv, public glint::util::DefinesPtrs<EGLGEnv>
{
public:
    EGLGEnv();
    virtual ~EGLGEnv() noexcept;

    virtual void shutdown();

    virtual int max_display_width() const noexcept;
    virtual int max_display_height() const noexcept;

    virtual GraphicsDisplay* create_display(int width, int height, int xpos, int ypos);

    virtual std::string name() const;
    virtual int major_version() const noexcept;
    virtual int minor_version() const noexcept;

    virtual int gl_major_version() const noexcept;
    virtual int gl_minor_version() const noexcept;

    void make_current();
    virtual void release_current();
    void swap_buffers();

private:
    void throw_if_wrong_EGL_version();
    void terminate_egl(EGLDisplay);
    void create_egl_context();
    void destroy_egl_context(EGLContext);
    void create_egl_surface();
    void destroy_egl_surface(EGLContext);
    void throw_if_wrong_GLES_version();

    genv::egl::internal::EGLGraphicsDisplay::UPtr gdisplay_;

    genv::internal::AbstractWEnv::UPtr wenv_;

    util::ResourcePtr<EGLDisplay, std::function<void(EGLDisplay)>> egl_display_;
    util::ResourcePtr<EGLContext, std::function<void(EGLContext)>> context_;
    util::ResourcePtr<EGLSurface, std::function<void(EGLSurface)>> surface_;

    EGLint egl_major_;
    EGLint egl_minor_;
    EGLConfig egl_config_;
    EGLint visual_id_;

    int gl_major_;
    int gl_minor_;

    genv::ConcreteWindow* win_;
};

} // namespace internal

} // namespace egl

namespace internal
{

extern AbstractGEnv::UPtr create_concrete_genv();

} // namespace internal

} // namespace genv

} // namespace glint

#endif
