/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_GENV_GLX_INTERNAL_GLXGENV_H
#define GLINT_GENV_GLX_INTERNAL_GLXGENV_H

#include <glint/genv/internal/AbstractGEnv.h>
#include <glint/genv/glx/internal/GLXGraphicsDisplay.h>
#include <glint/genv/x11/X11Window.h>

namespace glint
{

namespace genv
{

namespace glx
{

namespace internal
{

class GLXGEnv : public glint::genv::internal::AbstractGEnv, public glint::util::DefinesPtrs<GLXGEnv>
{
public:
    GLXGEnv();
    virtual ~GLXGEnv() noexcept;

    virtual void shutdown();

    virtual int max_display_width() const noexcept;
    virtual int max_display_height() const noexcept;

    virtual GraphicsDisplay* create_display(int width, int height, int xpos, int ypos);

    virtual std::string name() const;
    virtual int major_version() const noexcept;
    virtual int minor_version() const noexcept;

    virtual int gl_major_version() const noexcept;
    virtual int gl_minor_version() const noexcept;

    void make_current();
    void release_current();
    void swap_buffers();

private:
    void throw_if_wrong_GLX_version();
    void setup_visual();
    void create_glx_context();
    void destroy_glx_context(GLXContext c);
    void throw_if_wrong_GL_version();

    genv::glx::internal::GLXGraphicsDisplay::UPtr gdisplay_;

    genv::internal::AbstractWEnv::UPtr wenv_;

    util::ResourcePtr<GLXContext, std::function<void(GLXContext)>> context_;

    int glx_major_;
    int glx_minor_;

    typedef std::unique_ptr<
    XVisualInfo,
    decltype(&XFree)
    > VisualPtr;
    VisualPtr vi_;

    int gl_major_;
    int gl_minor_;

    x11::X11Window* win_;
};

} // namespace internal

} // namespace glx

namespace internal
{

extern AbstractGEnv::UPtr create_concrete_genv();

} // namespace internal

} // namespace genv

} // namespace glint

#endif
