/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_GENV_GLX_INTERNAL_GLXGRAHPICSDISPLAY_H
#define GLINT_GENV_GLX_INTERNAL_GLXGRAHPICSDISPLAY_H

#include <glint/GraphicsDisplay.h>
#include <glint/genv/internal/AbstractGEnv.h>
#include <glint/genv/internal/AbstractWEnv.h>
#include <glint/util/DefinesPtrs.h>
#include <glint/util/ResourcePtr.h>

#include <GL/glx.h>

namespace glint
{

namespace genv
{

namespace glx
{

namespace internal
{

class GLXGraphicsDisplay : public glint::GraphicsDisplay, public glint::util::DefinesPtrs<GLXGraphicsDisplay>
{
public:
    GLXGraphicsDisplay(glint::genv::internal::AbstractGEnv& genv, glint::genv::internal::AbstractWEnv& wenv);
    virtual ~GLXGraphicsDisplay() noexcept;

    virtual int gl_major_version() const noexcept;
    virtual int gl_minor_version() const noexcept;

    virtual int width() const noexcept;       // Actual height as opposed to requested height
    virtual int height() const noexcept;      // Actual width as opposed to requested width
    virtual int xpos() const noexcept;        // Actual x position as opposed to requested position
    virtual int ypos() const noexcept;        // Actual y position as opposed to requested position

    virtual void make_current();              // Assign the drawing context to the calling thread
    virtual void release_current();           // Release the drawing context
    virtual void swap_buffers();              // Swap the rendering buffers

    virtual Vis set_visibility(Vis v);        // Show or hide the window
    virtual Vis visibility();                 // Current visibility state

    virtual void set_stacking_order(StackingOrder order);  // Raise or lower the window

    virtual AbstractWindow* window() const noexcept;  // Returns the associated window

private:
    glint::genv::internal::AbstractGEnv& genv_;
    glint::genv::internal::AbstractWEnv& wenv_;
};

} // namespace internal

} // namespace glx

} // namespace genv

} // namespace glint

#endif
