/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_GENV_INTERNAL_ABSTRACTWENV_H
#define GLINT_GENV_INTERNAL_ABSTRACTWENV_H

#include <glint/util/DefinesPtrs.h>
#include <glint/genv/NativeDefs.h>

#include <boost/utility.hpp>

namespace glint
{

class AbstractWindow;

namespace genv
{

namespace internal
{

//
// Abstraction of a native windowing environment, such as X11.
//

class AbstractWEnv : public glint::util::DefinesPtrs<AbstractWEnv>, private boost::noncopyable
{
public:
    AbstractWEnv();
    virtual ~AbstractWEnv() noexcept;

    virtual void shutdown() = 0;

    virtual NativeDisplayType display() const noexcept = 0;

    virtual int max_display_width() const noexcept = 0;
    virtual int max_display_height() const noexcept = 0;

    virtual AbstractWindow* create_window(int visual_id, int width, int height, int xpos = 0, int ypos = 0) = 0;
    virtual AbstractWindow* window() const noexcept = 0;

    virtual int width() const noexcept = 0;  // Actual width and height, which may differ from requested values
    virtual int height() const noexcept = 0;

    virtual int xpos() const noexcept = 0;   // Actual x and y positions, which may differ from requested positions
    virtual int ypos() const noexcept = 0;

    enum class Vis { invisible, visible };
    virtual Vis set_visibility(Vis v) = 0;   // Show or hide the window
    virtual Vis visibility() = 0;            // Current visibility state

    enum class StackingOrder { bottom, top };
    virtual void set_stacking_order(StackingOrder order) = 0;  // Raise or lower the window
};

AbstractWEnv::UPtr create_concrete_wenv(); // Factory that instantiates the concrete version, such as X11.

} // namespace internal

} // namespace genv

} // namespace glint

#endif
