/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_GENV_X11_INTERNAL_X11WENV_H
#define GLINT_GENV_X11_INTERNAL_X11WENV_H

#include <glint/genv/internal/AbstractWEnv.h>
#include <glint/util/ResourcePtr.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

namespace glint
{

namespace genv
{

namespace x11
{

namespace internal
{

class X11WEnv : public glint::genv::internal::AbstractWEnv
{
public:
    X11WEnv();
    virtual ~X11WEnv() noexcept;

    virtual void shutdown();

    virtual NativeDisplayType display() const noexcept;

    virtual int max_display_width() const noexcept;
    virtual int max_display_height() const noexcept;

    virtual AbstractWindow* create_window(int visual_id, int width, int height, int xpos, int ypos);
    virtual AbstractWindow* window() const noexcept;

    virtual int width() const noexcept;   // Actual width and height, which may differ from requested values
    virtual int height() const noexcept;

    virtual int xpos() const noexcept;    // Actual x and y positions, which may differ from requested values
    virtual int ypos() const noexcept;

    virtual Vis set_visibility(Vis v);   // Show or hide the window
    virtual Vis visibility();            // Current visibility state

    virtual void set_stacking_order(StackingOrder order);  // Raise or lower the window

private:
    typedef std::unique_ptr<
        Display,
        decltype(&XCloseDisplay)
    > DisplayPtr;
    DisplayPtr display_;

    int max_width_;
    int max_height_;

    typedef std::unique_ptr<
        XVisualInfo,
        std::function<void(XVisualInfo*)>
    > VisualPtr;
    VisualPtr vi_;

    typedef glint::util::ResourcePtr<
        Colormap,
        std::function<void(Colormap)>
    > ColormapPtr;
    ColormapPtr cmap_;

    typedef glint::util::ResourcePtr<
        Window,
        std::function<void(Window)>
    > WindowPtr;
    WindowPtr x11win_;

    std::unique_ptr<glint::AbstractWindow> win_;

    void connect_display();

    void free_xvisual(XVisualInfo* vi);

    void free_colormap(Colormap cm);

    void destroy_window(Window w);

    void set_screen_size();
};

} // namespace internal

} // namespace x11

} // namespace genv

} // namespace glint

#endif
