/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_INTERNAL_EXCEPTIONIMPL_H
#define GLINT_INTERNAL_EXCEPTIONIMPL_H

#include <boost/utility.hpp>

#include <string>
#include <exception>

namespace glint
{

class Exception;

namespace internal
{

class ExceptionImpl : private boost::noncopyable
{
public:
    explicit ExceptionImpl(const std::string& reason);

    virtual const char* what() const noexcept = 0;
    virtual std::string reason() const;
    virtual std::string to_string(const std::nested_exception& nested, int indent_level, const std::string& indent) const;

    std::exception_ptr remember(const glint::Exception* env, std::exception_ptr earlier_exception);
    std::exception_ptr remember(const glint::Exception* env, const Exception& earlier_exception);
    std::exception_ptr get_earlier() const noexcept;

protected:
    virtual ~ExceptionImpl() noexcept;

private:
    const std::string reason_;
    std::exception_ptr previous_;
};

} // namespace internal

} // namespace glint

#endif
