/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_INTERNAL_GLINTEXCEPTIONSIMPL_H
#define GLINT_INTERNAL_GLINTEXCEPTIONSIMPL_H

#include <glint/internal/ExceptionImpl.h>

#include <boost/utility.hpp>

#include <string>
#include <exception>

namespace glint
{

class Exception;

namespace internal
{

class OpenGLExceptionImpl : public ExceptionImpl
{
public:
    OpenGLExceptionImpl(const std::string& reason, int gl_error);

    virtual const char* what() const noexcept;
    int gl_error() const noexcept;

private:
    const int gl_error_;
};

class InvalidArgumentExceptionImpl : public ExceptionImpl
{
public:
    InvalidArgumentExceptionImpl(const std::string& reason);

    virtual const char* what() const noexcept;
};

class LogicExceptionImpl : public ExceptionImpl
{
public:
    LogicExceptionImpl(const std::string& reason);

    virtual const char* what() const noexcept;
};

class ShutdownExceptionImpl : public ExceptionImpl
{
public:
    ShutdownExceptionImpl(const std::string& reason);

    virtual const char* what() const noexcept;
};

class FileExceptionImpl : public ExceptionImpl
{
public:
    FileExceptionImpl(const std::string& reason, int err);

    virtual const char* what() const noexcept;
    int error() const noexcept;

private:
    int errno_;
};

} // namespace internal

} // namespace glint

#endif
