/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_INTERNAL_RUNTIMEIMPL_H
#define GLINT_INTERNAL_RUNTIMEIMPL_H

#include <glint/Runtime.h>
#include <glint/genv/internal/AbstractGEnv.h>
#include <glint/GraphicsDisplay.h>

#include <memory>
#include <atomic>
#include <limits>

namespace glint
{

namespace internal
{

//
// Nux Runtime implementation
//
class RuntimeImpl : private boost::noncopyable
{
public:
    RuntimeImpl();
    ~RuntimeImpl() noexcept;

    void shutdown();

    int max_display_width() const noexcept;
    int max_display_height() const noexcept;

    GraphicsDisplay* create_display();
    GraphicsDisplay* create_display(int width, int height, int xpos = 0, int ypos = 0);
    GraphicsDisplay* display() const;

private:
    void init_GL();

    mutable std::atomic_flag display_created_;
    GraphicsDisplay* display_;
    std::atomic_flag finalized_;
    glint::genv::internal::AbstractGEnv::UPtr genv_;
};

} // namespace internal

} // namespace glint

#endif
