/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_INTERNAL_SHADERIMPL_H
#define GLINT_INTERNAL_SHADERIMPL_H

#include <glint/util/ResourcePtr.h>

#include <boost/utility.hpp>
#include <glint/OpenGL.h>

#include <string>

namespace glint
{

namespace internal
{

class ShaderImpl
{
public:
    static bool supports_compiled_shaders();

private:
    ShaderImpl() = default;
    ~ShaderImpl() = default;
};

class VertexShaderImpl : private boost::noncopyable
{
public:
    explicit VertexShaderImpl(const std::string& source);
    ~VertexShaderImpl() = default;

    inline GLuint id() const noexcept;

private:
    util::ResourcePtr<GLuint, std::function<void(GLuint)>> id_;
};

class FragmentShaderImpl : private boost::noncopyable
{
public:
    explicit FragmentShaderImpl(const std::string& source);
    ~FragmentShaderImpl() = default;

    inline GLuint id() const noexcept;

private:
    util::ResourcePtr<GLuint, std::function<void(GLuint)>> id_;
};

GLuint
VertexShaderImpl::
id() const noexcept
{
    return id_.get();
}

GLuint
FragmentShaderImpl::
id() const noexcept
{
    return id_.get();
}

} // namespace internal

} // namespace glint

#endif
