/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_INTERNAL_SHADERVARIABLEIMPL_H
#define GLINT_INTERNAL_SHADERVARIABLEIMPL_H

#include <glint/OpenGL.h>
#include <boost/utility.hpp>

#include <memory>
#include <string>

namespace glint
{

namespace internal
{

class ShaderVariableImpl : private boost::noncopyable
{
public:
    inline GLuint program_id() const noexcept;
    inline GLint location() const noexcept;
    inline std::string name() const noexcept;
    inline GLenum type() const noexcept;
    inline GLint size() const noexcept;

protected:
    ShaderVariableImpl(GLuint prog_id, GLint location, const std::string& name, GLenum type, GLint size) noexcept;
    virtual ~ShaderVariableImpl();

    void check(const char* op) const;
    void check_get() const;
    void check_set() const;
    void check_size(size_t vector_size) const;

private:
    GLuint prog_id_;
    GLint location_;
    std::string name_;
    GLenum type_;
    GLint size_;
};

GLuint
ShaderVariableImpl::
program_id() const noexcept
{
    return prog_id_;
}

GLint
ShaderVariableImpl::
location() const noexcept
{
    return location_;
}

std::string
ShaderVariableImpl::
name() const noexcept
{
    return name_;
}

GLenum
ShaderVariableImpl::
type() const noexcept
{
    return type_;
}

GLint
ShaderVariableImpl::
size() const noexcept
{
    return size_;
}

} // namespace internal

} // namespace glint

#endif
