/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_INTERNAL_UNIFORMVARIABLEIMPL_H
#define GLINT_INTERNAL_UNIFORMVARIABLEIMPL_H

#include <glint/internal/ShaderVariableImpl.h>
#include <glint/Types.h>

namespace glint
{

namespace internal
{

class UniformVariableImpl : public ShaderVariableImpl
{
protected:
    UniformVariableImpl(GLuint prog_id, GLint location, const std::string& name, GLenum type, GLint size);
    virtual ~UniformVariableImpl() noexcept;
};

class IntUniformVariableBaseImpl : public UniformVariableImpl
{
public:
    IntUniformVariableBaseImpl(GLuint prog_id, GLint location,
                               const std::string& name, GLenum type, GLint size);
    ~IntUniformVariableBaseImpl() noexcept;

    GLint get() const;
    void set(GLint v);
};

class IntArrayUniformVariableBaseImpl : public UniformVariableImpl
{
public:
    IntArrayUniformVariableBaseImpl(GLuint prog_id, GLint location,
                                    const std::string& name, GLenum type, GLint size);
    ~IntArrayUniformVariableBaseImpl() noexcept;

    std::vector<GLint> get() const;
    void set(const std::vector<GLint>& v);
};

class FloatUniformImpl : public UniformVariableImpl
{
public:
    FloatUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    ~FloatUniformImpl() noexcept;

    GLfloat get() const;
    void set(GLfloat v);
};

class FloatVec2UniformImpl : public UniformVariableImpl
{
public:
    FloatVec2UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatVec2UniformImpl() noexcept;

    FloatVec2 get() const;
    void set(const FloatVec2& v);
};

class FloatVec3UniformImpl : public UniformVariableImpl
{
public:
    FloatVec3UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatVec3UniformImpl() noexcept;

    FloatVec3 get() const;
    void set(const FloatVec3& v);
};

class FloatVec4UniformImpl : public UniformVariableImpl
{
public:
    FloatVec4UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatVec4UniformImpl() noexcept;

    FloatVec4 get() const;
    void set(const FloatVec4& v);
};

class FloatMat2UniformImpl : public UniformVariableImpl
{
public:
    FloatMat2UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatMat2UniformImpl() noexcept;

    FloatMat2 get() const;
    void set(const FloatMat2& v);
};

class FloatMat3UniformImpl : public UniformVariableImpl
{
public:
    FloatMat3UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatMat3UniformImpl() noexcept;

    FloatMat3 get() const;
    void set(const FloatMat3& v);
};

class FloatMat4UniformImpl : public UniformVariableImpl
{
public:
    FloatMat4UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatMat4UniformImpl() noexcept;

    FloatMat4 get() const;
    void set(const FloatMat4& v);
};

class FloatArrayUniformImpl : public UniformVariableImpl
{
public:
    FloatArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatArrayUniformImpl() noexcept;

    FloatArray get() const;
    void set(const FloatArray& v);
};

class FloatVec2ArrayUniformImpl : public UniformVariableImpl
{
public:
    FloatVec2ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatVec2ArrayUniformImpl() noexcept;

    FloatVec2Array get() const;
    void set(const FloatVec2Array& v);
};

class FloatVec3ArrayUniformImpl : public UniformVariableImpl
{
public:
    FloatVec3ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatVec3ArrayUniformImpl() noexcept;

    FloatVec3Array get() const;
    void set(const FloatVec3Array& v);
};

class FloatVec4ArrayUniformImpl : public UniformVariableImpl
{
public:
    FloatVec4ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatVec4ArrayUniformImpl() noexcept;

    FloatVec4Array get() const;
    void set(const FloatVec4Array& v);
};

class FloatMat2ArrayUniformImpl : public UniformVariableImpl
{
public:
    FloatMat2ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatMat2ArrayUniformImpl() noexcept;

    FloatMat2Array get() const;
    void set(const FloatMat2Array& v);
};

class FloatMat3ArrayUniformImpl : public UniformVariableImpl
{
public:
    FloatMat3ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatMat3ArrayUniformImpl() noexcept;

    FloatMat3Array get() const;
    void set(const FloatMat3Array& v);
};

class FloatMat4ArrayUniformImpl : public UniformVariableImpl
{
public:
    FloatMat4ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~FloatMat4ArrayUniformImpl() noexcept;

    FloatMat4Array get() const;
    void set(const FloatMat4Array& v);
};

class IntUniformImpl : public IntUniformVariableBaseImpl
{
public:
    IntUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~IntUniformImpl() noexcept;
};

class IntVec2UniformImpl : public UniformVariableImpl
{
public:
    IntVec2UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~IntVec2UniformImpl() noexcept;

    IntVec2 get() const;
    void set(const IntVec2& v);
};

class IntVec3UniformImpl : public UniformVariableImpl
{
public:
    IntVec3UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~IntVec3UniformImpl() noexcept;

    IntVec3 get() const;
    void set(const IntVec3& v);
};

class IntVec4UniformImpl : public UniformVariableImpl
{
public:
    IntVec4UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~IntVec4UniformImpl() noexcept;

    IntVec4 get() const;
    void set(const IntVec4& v);
};

class IntArrayUniformImpl : public IntArrayUniformVariableBaseImpl
{
public:
    IntArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~IntArrayUniformImpl() noexcept;
};

class IntVec2ArrayUniformImpl : public UniformVariableImpl
{
public:
    IntVec2ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~IntVec2ArrayUniformImpl() noexcept;

    IntVec2Array get() const;
    void set(const IntVec2Array& v);
};

class IntVec3ArrayUniformImpl : public UniformVariableImpl
{
public:
    IntVec3ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~IntVec3ArrayUniformImpl() noexcept;

    IntVec3Array get() const;
    void set(const IntVec3Array& v);
};

class IntVec4ArrayUniformImpl : public UniformVariableImpl
{
public:
    IntVec4ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~IntVec4ArrayUniformImpl() noexcept;

    IntVec4Array get() const;
    void set(const IntVec4Array& v);
};

class BoolUniformImpl : public UniformVariableImpl
{
public:
    BoolUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    ~BoolUniformImpl() noexcept;

    GLboolean get() const;
    void set(GLboolean v);
};

class BoolVec2UniformImpl : public UniformVariableImpl
{
public:
    BoolVec2UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~BoolVec2UniformImpl() noexcept;

    BoolVec2 get() const;
    void set(const BoolVec2& v);
};

class BoolVec3UniformImpl : public UniformVariableImpl
{
public:
    BoolVec3UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~BoolVec3UniformImpl() noexcept;

    BoolVec3 get() const;
    void set(const BoolVec3& v);
};

class BoolVec4UniformImpl : public UniformVariableImpl
{
public:
    BoolVec4UniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~BoolVec4UniformImpl() noexcept;

    BoolVec4 get() const;
    void set(const BoolVec4& v);
};

class BoolArrayUniformImpl : public UniformVariableImpl
{
public:
    BoolArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~BoolArrayUniformImpl() noexcept;

    BoolArray get() const;
    void set(const BoolArray& v);
};

class BoolVec2ArrayUniformImpl : public UniformVariableImpl
{
public:
    BoolVec2ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~BoolVec2ArrayUniformImpl() noexcept;

    BoolVec2Array get() const;
    void set(const BoolVec2Array& v);
};

class BoolVec3ArrayUniformImpl : public UniformVariableImpl
{
public:
    BoolVec3ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~BoolVec3ArrayUniformImpl() noexcept;

    BoolVec3Array get() const;
    void set(const BoolVec3Array& v);
};

class BoolVec4ArrayUniformImpl : public UniformVariableImpl
{
public:
    BoolVec4ArrayUniformImpl(GLuint prog_id, GLint location, const std::string& name, GLint size);
    virtual ~BoolVec4ArrayUniformImpl() noexcept;

    BoolVec4Array get() const;
    void set(const BoolVec4Array& v);
};

class Sampler2DUniformImpl : public IntUniformVariableBaseImpl
{
public:
    Sampler2DUniformImpl(GLuint prog_id, GLint location,
                         const std::string& name, GLint size);
    ~Sampler2DUniformImpl() noexcept;
};

class Sampler2DArrayUniformImpl : public IntArrayUniformVariableBaseImpl
{
public:
    Sampler2DArrayUniformImpl(GLuint prog_id, GLint location,
                              const std::string& name, GLint size);
    virtual ~Sampler2DArrayUniformImpl() noexcept;
};

class SamplerCubeUniformImpl : public IntUniformVariableBaseImpl
{
public:
    SamplerCubeUniformImpl(GLuint prog_id, GLint location,
                           const std::string& name, GLint size);
    ~SamplerCubeUniformImpl() noexcept;
};

class SamplerCubeArrayUniformImpl : public IntArrayUniformVariableBaseImpl
{
public:
    SamplerCubeArrayUniformImpl(GLuint prog_id, GLint location,
                                const std::string& name, GLint size);
    virtual ~SamplerCubeArrayUniformImpl() noexcept;
};

} // namespace internal

} // namespace glint

#endif
