/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#ifndef GLINT_UTIL_DEFINESPTRS_H
#define GLINT_UTIL_DEFINESPTRS_H

#include <memory>

namespace glint
{

namespace util
{

//
// Simple template class to inject typedefs for the various smart pointers into a class.
//
// Use like this:
//
// class MyClass : public util::DefinesPtrs<MyClass> {
//   // MyClass now provides public typedefs for Ptr, CPtr, UPtr, and UCPtr.
//   ...
// };
//

template <typename T>
class DefinesPtrs
{
public:
    typedef std::shared_ptr<T> Ptr;
    typedef std::shared_ptr<const T> CPtr;
    typedef std::unique_ptr<T> UPtr;
    typedef std::unique_ptr<const T> UCPtr;

protected:                                  // Not meant to be instantiated stand-alone
    DefinesPtrs() = default;
    virtual ~DefinesPtrs() {};
};

} // namespace util

} // namespace glint

#endif
