/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/GLUtil.h>
#include <glint/GlintExceptions.h>

#include <sstream>
#include <cassert>

using namespace std;

namespace glint
{

string
gl_err_to_name(GLenum gl_err)
{
    switch (gl_err)
    {
        case GL_NO_ERROR:
        {
            return "GL_NO_ERROR";
            break;
        }
        case GL_INVALID_ENUM:
        {
            return "GL_INVALID_ENUM";
            break;
        }
        case GL_INVALID_VALUE:
        {
            return "GL_INVALID_VALUE";
            break;
        }
        case GL_INVALID_OPERATION:
        {
            return "GL_INVALID_OPERATION";
            break;
        }
        case GL_OUT_OF_MEMORY:
        {
            return "GL_OUT_OF_MEMORY";
            break;
        }
        case GL_INVALID_FRAMEBUFFER_OPERATION:
        {
            return "GL_INVALID_FRAMEBUFFER_OPERATION";
            break;
        }
        default:
        {
            ostringstream s;
            s << "Unknown GL error: " << gl_err << " (0x" << hex << gl_err << ")";
            return s.str();
            break;
        }
    }
    assert(false);
    return "";      // Suppress compiler warning.
}

void
check_gl(const string& msg)
{
    GLenum err;
    exception_ptr ep;
    // We need to call glGetError() in a loop until it returns "no error" (as of OpenGL 2.0).
    while ((err = glGetError()) != GL_NO_ERROR)
    {
        OpenGLException e(msg, err);
        e.remember(ep);
        ep = make_exception_ptr(e);
    }
    if (ep)
    {
        rethrow_exception(ep);
    }
}

GLenum
clear_gl_error()
{
    GLenum err = glGetError();
    while (glGetError() != GL_NO_ERROR)     // Keep calling until all the flags are cleared
        ;
    return err; // Return initial error
}

} // namespace glint
