/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/GlintExceptions.h>
#include <glint/internal/GlintExceptionsImpl.h>

using namespace std;

namespace glint
{

OpenGLException::
OpenGLException(const string& reason, int gl_error)
    : Exception(make_shared<internal::OpenGLExceptionImpl>(reason, gl_error))
{
}

OpenGLException::
OpenGLException(const OpenGLException&) = default;

OpenGLException&
OpenGLException::
operator=(const OpenGLException&) = default;

OpenGLException::
~OpenGLException() noexcept = default;

const char*
OpenGLException::
what() const noexcept
{
    return dynamic_pointer_cast<internal::OpenGLExceptionImpl>(pimpl())->what();
}

int
OpenGLException::
gl_error() const noexcept
{
    return dynamic_pointer_cast<internal::OpenGLExceptionImpl>(pimpl())->gl_error();
}

exception_ptr
OpenGLException::
self() const
{
    return make_exception_ptr(*this);
}

InvalidArgumentException::
InvalidArgumentException(const string& reason)
    : Exception(make_shared<internal::InvalidArgumentExceptionImpl>(reason))
{
}

InvalidArgumentException::
InvalidArgumentException(const InvalidArgumentException&) = default;

InvalidArgumentException&
InvalidArgumentException::
operator=(const InvalidArgumentException&) = default;

InvalidArgumentException::
~InvalidArgumentException() noexcept = default;

const char*
InvalidArgumentException::
what() const noexcept
{
    return dynamic_pointer_cast<internal::InvalidArgumentExceptionImpl>(pimpl())->what();
}

exception_ptr
InvalidArgumentException::
self() const
{
    return make_exception_ptr(*this);
}

LogicException::
LogicException(const string& reason)
    : Exception(make_shared<internal::LogicExceptionImpl>(reason))
{
}

LogicException::
LogicException(const LogicException&) = default;

LogicException&
LogicException::
operator=(const LogicException&) = default;

LogicException::
~LogicException() noexcept = default;

const char*
LogicException::
what() const noexcept
{
    return dynamic_pointer_cast<internal::LogicExceptionImpl>(pimpl())->what();
}

exception_ptr
LogicException::
self() const
{
    return make_exception_ptr(*this);
}

ShutdownException::
ShutdownException(const string& reason)
    : Exception(make_shared<internal::ShutdownExceptionImpl>(reason))
{
}

ShutdownException::
ShutdownException(const ShutdownException&) = default;

ShutdownException&
ShutdownException::
operator=(const ShutdownException&) = default;

ShutdownException::
~ShutdownException() noexcept = default;

const char*
ShutdownException::
what() const noexcept
{
    return dynamic_pointer_cast<internal::ShutdownExceptionImpl>(pimpl())->what();
}

exception_ptr
ShutdownException::
self() const
{
    return make_exception_ptr(*this);
}

FileException::
FileException(const string& reason, int err)
    : Exception(make_shared<internal::FileExceptionImpl>(reason, err))
{
}

FileException::
FileException(const FileException&) = default;

FileException&
FileException::
operator=(const FileException&) = default;

FileException::
~FileException() noexcept = default;

const char*
FileException::
what() const noexcept
{
    return dynamic_pointer_cast<internal::FileExceptionImpl>(pimpl())->what();
}

int
FileException::
error() const noexcept
{
    return dynamic_pointer_cast<internal::FileExceptionImpl>(pimpl())->error();
}

exception_ptr
FileException::
self() const
{
    return make_exception_ptr(*this);
}

} // namespace glint
