/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/Program.h>
#include <glint/internal/ProgramImpl.h>


using namespace glint;
using namespace std;

namespace glint
{

Program::UPtr
Program::
create(const VertexShader& vs, const FragmentShader& fs)
{
    return UPtr(new Program(vs, fs));
}

GLuint
Program::
id() const noexcept
{
    return p_->id();
}

void
Program::
use() const
{
    return p_->use();
}

bool
Program::
in_use() const noexcept
{
    return p_->in_use();
}

GLuint
Program::
vertex_shader_id() const
{
    return p_->vertex_shader_id();
}

GLuint
Program::
fragment_shader_id() const
{
    return p_->fragment_shader_id();
}

UniformVariable&
Program::
uniform(const string& name) const
{
    return p_->uniform(name);
}

const Program::UniformMap&
Program::
uniforms() const
{
    return p_->uniforms();
}

Program::
Program(const VertexShader& vs, const FragmentShader& fs)
    : p_(make_shared<internal::ProgramImpl>(vs, fs))
{
}

Program::
~Program() noexcept
{
}

} // namespace glint
