/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/Shader.h>
#include <glint/internal/ShaderImpl.h>
#include <glint/util/FileIO.h>
#include <glint/GLUtil.h>


using namespace glint;
using namespace std;

namespace glint
{

Shader::
Shader() = default;

Shader::
~Shader() = default;

bool
Shader::
supports_compiled_shaders()
{
    return internal::ShaderImpl::supports_compiled_shaders();
}

VertexShader::UPtr
VertexShader::
create(const std::string& source)
{
    return UPtr(new VertexShader(source));
}

VertexShader::UPtr
VertexShader::
create_from_file(const std::string& filename)
{
    return UPtr(new VertexShader(util::read_text_file(filename)));
}

VertexShader::
VertexShader(const string& source)
    : p_(make_shared<internal::VertexShaderImpl>(source))
{
}

VertexShader::
~VertexShader() noexcept
{
}

GLuint
VertexShader::
id() const noexcept
{
    return p_->id();
}

FragmentShader::UPtr
FragmentShader::
create(const std::string& source)
{
    return UPtr(new FragmentShader(source));
}

FragmentShader::UPtr
FragmentShader::
create_from_file(const std::string& filename)
{
    return UPtr(new FragmentShader(util::read_text_file(filename)));
}

FragmentShader::
FragmentShader(const string& source)
    : p_(make_shared<internal::FragmentShaderImpl>(source))
{
}

FragmentShader::
~FragmentShader() noexcept
{
}

GLuint
FragmentShader::
id() const noexcept
{
    return p_->id();
}

} // namespace glint
