/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/UniformVariable.h>
#include <glint/internal/UniformVariableImpl.h>

using namespace glint;
using namespace std;

namespace glint
{

// UniformVariable

UniformVariable::
UniformVariable(const shared_ptr<internal::UniformVariableImpl>& derived)
    : ShaderVariable(derived)
{
}

UniformVariable::
~UniformVariable() = default;

void
UniformVariable::
set(GLfloat rhs)
{
    FloatUniform& u = dynamic_cast<FloatUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(GLfloat rhs)
{
    FloatUniform& u = dynamic_cast<FloatUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatVec2& rhs)
{
    FloatVec2Uniform& u = dynamic_cast<FloatVec2Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatVec2& rhs)
{
    FloatVec2Uniform& u = dynamic_cast<FloatVec2Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatVec3& rhs)
{
    FloatVec3Uniform& u = dynamic_cast<FloatVec3Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatVec3& rhs)
{
    FloatVec3Uniform& u = dynamic_cast<FloatVec3Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatVec4& rhs)
{
    FloatVec4Uniform& u = dynamic_cast<FloatVec4Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatVec4& rhs)
{
    FloatVec4Uniform& u = dynamic_cast<FloatVec4Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatMat2& rhs)
{
    FloatMat2Uniform& u = dynamic_cast<FloatMat2Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatMat2& rhs)
{
    FloatMat2Uniform& u = dynamic_cast<FloatMat2Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatMat3& rhs)
{
    FloatMat3Uniform& u = dynamic_cast<FloatMat3Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatMat3& rhs)
{
    FloatMat3Uniform& u = dynamic_cast<FloatMat3Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatMat4& rhs)
{
    FloatMat4Uniform& u = dynamic_cast<FloatMat4Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatMat4& rhs)
{
    FloatMat4Uniform& u = dynamic_cast<FloatMat4Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatArray& rhs)
{
    FloatArrayUniform& u = dynamic_cast<FloatArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatArray& rhs)
{
    FloatArrayUniform& u = dynamic_cast<FloatArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatVec2Array& rhs)
{
    FloatVec2ArrayUniform& u = dynamic_cast<FloatVec2ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatVec2Array& rhs)
{
    FloatVec2ArrayUniform& u = dynamic_cast<FloatVec2ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatVec3Array& rhs)
{
    FloatVec3ArrayUniform& u = dynamic_cast<FloatVec3ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatVec3Array& rhs)
{
    FloatVec3ArrayUniform& u = dynamic_cast<FloatVec3ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatVec4Array& rhs)
{
    FloatVec4ArrayUniform& u = dynamic_cast<FloatVec4ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatVec4Array& rhs)
{
    FloatVec4ArrayUniform& u = dynamic_cast<FloatVec4ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatMat2Array& rhs)
{
    FloatMat2ArrayUniform& u = dynamic_cast<FloatMat2ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatMat2Array& rhs)
{
    FloatMat2ArrayUniform& u = dynamic_cast<FloatMat2ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatMat3Array& rhs)
{
    FloatMat3ArrayUniform& u = dynamic_cast<FloatMat3ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatMat3Array& rhs)
{
    FloatMat3ArrayUniform& u = dynamic_cast<FloatMat3ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const FloatMat4Array& rhs)
{
    FloatMat4ArrayUniform& u = dynamic_cast<FloatMat4ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const FloatMat4Array& rhs)
{
    FloatMat4ArrayUniform& u = dynamic_cast<FloatMat4ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(GLint rhs)
{
    IntUniformVariableBase& u = dynamic_cast<IntUniformVariableBase&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(GLint rhs)
{
    IntUniformVariableBase& u = dynamic_cast<IntUniformVariableBase&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const IntVec2& rhs)
{
    IntVec2Uniform& u = dynamic_cast<IntVec2Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const IntVec2& rhs)
{
    IntVec2Uniform& u = dynamic_cast<IntVec2Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const IntVec3& rhs)
{
    IntVec3Uniform& u = dynamic_cast<IntVec3Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const IntVec3& rhs)
{
    IntVec3Uniform& u = dynamic_cast<IntVec3Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const IntVec4& rhs)
{
    IntVec4Uniform& u = dynamic_cast<IntVec4Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const IntVec4& rhs)
{
    IntVec4Uniform& u = dynamic_cast<IntVec4Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const vector<GLint>& rhs)
{
    IntArrayUniformVariableBase& u = dynamic_cast<IntArrayUniformVariableBase&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const vector<GLint>& rhs)
{
    IntArrayUniformVariableBase& u = dynamic_cast<IntArrayUniformVariableBase&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const IntVec2Array& rhs)
{
    IntVec2ArrayUniform& u = dynamic_cast<IntVec2ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const IntVec2Array& rhs)
{
    IntVec2ArrayUniform& u = dynamic_cast<IntVec2ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const IntVec3Array& rhs)
{
    IntVec3ArrayUniform& u = dynamic_cast<IntVec3ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const IntVec3Array& rhs)
{
    IntVec3ArrayUniform& u = dynamic_cast<IntVec3ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const IntVec4Array& rhs)
{
    IntVec4ArrayUniform& u = dynamic_cast<IntVec4ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const IntVec4Array& rhs)
{
    IntVec4ArrayUniform& u = dynamic_cast<IntVec4ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(GLboolean rhs)
{
    BoolUniform& u = dynamic_cast<BoolUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(GLboolean rhs)
{
    BoolUniform& u = dynamic_cast<BoolUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const BoolVec2& rhs)
{
    BoolVec2Uniform& u = dynamic_cast<BoolVec2Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const BoolVec2& rhs)
{
    BoolVec2Uniform& u = dynamic_cast<BoolVec2Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const BoolVec3& rhs)
{
    BoolVec3Uniform& u = dynamic_cast<BoolVec3Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const BoolVec3& rhs)
{
    BoolVec3Uniform& u = dynamic_cast<BoolVec3Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const BoolVec4& rhs)
{
    BoolVec4Uniform& u = dynamic_cast<BoolVec4Uniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const BoolVec4& rhs)
{
    BoolVec4Uniform& u = dynamic_cast<BoolVec4Uniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const BoolArray& rhs)
{
    BoolArrayUniform& u = dynamic_cast<BoolArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const BoolArray& rhs)
{
    BoolArrayUniform& u = dynamic_cast<BoolArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const BoolVec2Array& rhs)
{
    BoolVec2ArrayUniform& u = dynamic_cast<BoolVec2ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const BoolVec2Array& rhs)
{
    BoolVec2ArrayUniform& u = dynamic_cast<BoolVec2ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const BoolVec3Array& rhs)
{
    BoolVec3ArrayUniform& u = dynamic_cast<BoolVec3ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const BoolVec3Array& rhs)
{
    BoolVec3ArrayUniform& u = dynamic_cast<BoolVec3ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

void
UniformVariable::
set(const BoolVec4Array& rhs)
{
    BoolVec4ArrayUniform& u = dynamic_cast<BoolVec4ArrayUniform&>(*this);
    u.set(rhs);
}

UniformVariable&
UniformVariable::
operator=(const BoolVec4Array& rhs)
{
    BoolVec4ArrayUniform& u = dynamic_cast<BoolVec4ArrayUniform&>(*this);
    u.set(rhs);
    return u;
}

// IntUniformVariable

IntUniformVariableBase::
IntUniformVariableBase(const shared_ptr<internal::IntUniformVariableBaseImpl>& derived)
    : UniformVariable(derived)
{
}

IntUniformVariableBase::
~IntUniformVariableBase() = default;

GLint
IntUniformVariableBase::
get() const
{
    return dynamic_pointer_cast<internal::IntUniformVariableBaseImpl>(pimpl())->get();
}

void
IntUniformVariableBase::
set(GLint rhs)
{
    dynamic_pointer_cast<internal::IntUniformVariableBaseImpl>(pimpl())->set(rhs);
}

IntUniformVariableBase::
operator GLint() const
{
    return get();
}

// IntArrayUniformVariable

IntArrayUniformVariableBase::
IntArrayUniformVariableBase(const shared_ptr<internal::IntArrayUniformVariableBaseImpl>& derived)
    : UniformVariable(derived)
{
}

IntArrayUniformVariableBase::
~IntArrayUniformVariableBase() = default;

vector<GLint>
IntArrayUniformVariableBase::
get() const
{
    return dynamic_pointer_cast<internal::IntArrayUniformVariableBaseImpl>(pimpl())->get();
}

void
IntArrayUniformVariableBase::
set(const vector<GLint>& rhs)
{
    dynamic_pointer_cast<internal::IntArrayUniformVariableBaseImpl>(pimpl())->set(rhs);
}

IntArrayUniformVariableBase::
operator IntArray() const
{
    return get();
}

// Float

FloatUniform::
FloatUniform(GLuint prog_id, GLint location, const std::string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatUniformImpl>(prog_id, location, name, size))
{
}

FloatUniform::
~FloatUniform() = default;

GLfloat
FloatUniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatUniformImpl>(pimpl())->get();
}

void
FloatUniform::
set(GLfloat v)
{
    dynamic_pointer_cast<internal::FloatUniformImpl>(pimpl())->set(v);
}

FloatUniform::
operator GLfloat() const
{
    return get();
}

FloatUniform&
FloatUniform::
operator=(GLfloat rhs)
{
    set(rhs);
    return *this;
}

FloatUniform&
FloatUniform::
operator=(const FloatUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatVec2

FloatVec2Uniform::
FloatVec2Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatVec2UniformImpl>(prog_id, location, name, size))
{
}

FloatVec2Uniform::
~FloatVec2Uniform() = default;

FloatVec2
FloatVec2Uniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatVec2UniformImpl>(pimpl())->get();
}

void
FloatVec2Uniform::
get(GLfloat& x, GLfloat& y) const
{
    FloatVec2 v = dynamic_pointer_cast<internal::FloatVec2UniformImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
}

void
FloatVec2Uniform::
set(const FloatVec2& v)
{
    dynamic_pointer_cast<internal::FloatVec2UniformImpl>(pimpl())->set(v);
}

void
FloatVec2Uniform::
set(GLfloat x, GLfloat y)
{
    FloatVec2 v = { { x, y } };
    dynamic_pointer_cast<internal::FloatVec2UniformImpl>(pimpl())->set(v);
}

FloatVec2Uniform::
operator FloatVec2() const
{
    return get();
}

FloatVec2Uniform&
FloatVec2Uniform::
operator=(const FloatVec2& rhs)
{
    set(rhs);
    return *this;
}

FloatVec2Uniform&
FloatVec2Uniform::
operator=(const FloatVec2Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatVec3

FloatVec3Uniform::
FloatVec3Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatVec3UniformImpl>(prog_id, location, name, size))
{
}

FloatVec3Uniform::
~FloatVec3Uniform() = default;

FloatVec3
FloatVec3Uniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatVec3UniformImpl>(pimpl())->get();
}

void
FloatVec3Uniform::
get(GLfloat& x, GLfloat& y, GLfloat& z) const
{
    FloatVec3 v = dynamic_pointer_cast<internal::FloatVec3UniformImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
    z = v[2];
}

void
FloatVec3Uniform::
set(const FloatVec3& v)
{
    dynamic_pointer_cast<internal::FloatVec3UniformImpl>(pimpl())->set(v);
}

void
FloatVec3Uniform::
set(GLfloat x, GLfloat y, GLfloat z)
{
    FloatVec3 v = { { x, y, z } };
    dynamic_pointer_cast<internal::FloatVec3UniformImpl>(pimpl())->set(v);
}

FloatVec3Uniform::
operator FloatVec3() const
{
    return get();
}

FloatVec3Uniform&
FloatVec3Uniform::
operator=(const FloatVec3& rhs)
{
    set(rhs);
    return *this;
}

FloatVec3Uniform&
FloatVec3Uniform::
operator=(const FloatVec3Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatVec4

FloatVec4Uniform::
FloatVec4Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatVec4UniformImpl>(prog_id, location, name, size))
{
}

FloatVec4Uniform::
~FloatVec4Uniform() = default;

FloatVec4
FloatVec4Uniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatVec4UniformImpl>(pimpl())->get();
}

void
FloatVec4Uniform::
get(GLfloat& x, GLfloat& y, GLfloat& z, GLfloat& w) const
{
    FloatVec4 v = dynamic_pointer_cast<internal::FloatVec4UniformImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
    z = v[2];
    w = v[3];
}

void
FloatVec4Uniform::
set(const FloatVec4& v)
{
    dynamic_pointer_cast<internal::FloatVec4UniformImpl>(pimpl())->set(v);
}

void
FloatVec4Uniform::
set(GLfloat x, GLfloat y, GLfloat z, GLfloat w)
{
    FloatVec4 v = { { x, y, z, w } };
    dynamic_pointer_cast<internal::FloatVec4UniformImpl>(pimpl())->set(v);
}

FloatVec4Uniform::
operator FloatVec4() const
{
    return get();
}

FloatVec4Uniform&
FloatVec4Uniform::
operator=(const FloatVec4& rhs)
{
    set(rhs);
    return *this;
}

FloatVec4Uniform&
FloatVec4Uniform::
operator=(const FloatVec4Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatMat2

FloatMat2Uniform::
FloatMat2Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatMat2UniformImpl>(prog_id, location, name, size))
{
}

FloatMat2Uniform::
~FloatMat2Uniform() = default;

FloatMat2
FloatMat2Uniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatMat2UniformImpl>(pimpl())->get();
}

void
FloatMat2Uniform::
set(const FloatMat2& v)
{
    dynamic_pointer_cast<internal::FloatMat2UniformImpl>(pimpl())->set(v);
}

FloatMat2Uniform::
operator FloatMat2() const
{
    return get();
}

FloatMat2Uniform&
FloatMat2Uniform::
operator=(const FloatMat2& rhs)
{
    set(rhs);
    return *this;
}

FloatMat2Uniform&
FloatMat2Uniform::
operator=(const FloatMat2Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatMat3

FloatMat3Uniform::
FloatMat3Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatMat3UniformImpl>(prog_id, location, name, size))
{
}

FloatMat3Uniform::
~FloatMat3Uniform() = default;

FloatMat3
FloatMat3Uniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatMat3UniformImpl>(pimpl())->get();
}

void
FloatMat3Uniform::
set(const FloatMat3& v)
{
    dynamic_pointer_cast<internal::FloatMat3UniformImpl>(pimpl())->set(v);
}

FloatMat3Uniform::
operator FloatMat3() const
{
    return get();
}

FloatMat3Uniform&
FloatMat3Uniform::
operator=(const FloatMat3& rhs)
{
    set(rhs);
    return *this;
}

FloatMat3Uniform&
FloatMat3Uniform::
operator=(const FloatMat3Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatMat4

FloatMat4Uniform::
FloatMat4Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatMat4UniformImpl>(prog_id, location, name, size))
{
}

FloatMat4Uniform::
~FloatMat4Uniform() = default;

FloatMat4
FloatMat4Uniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatMat4UniformImpl>(pimpl())->get();
}

void
FloatMat4Uniform::
set(const FloatMat4& v)
{
    dynamic_pointer_cast<internal::FloatMat4UniformImpl>(pimpl())->set(v);
}

FloatMat4Uniform::
operator FloatMat4() const
{
    return get();
}

FloatMat4Uniform&
FloatMat4Uniform::
operator=(const FloatMat4& rhs)
{
    set(rhs);
    return *this;
}

FloatMat4Uniform&
FloatMat4Uniform::
operator=(const FloatMat4Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatArray

FloatArrayUniform::
FloatArrayUniform(GLuint prog_id, GLint location, const std::string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatArrayUniformImpl>(prog_id, location, name, size))
{
}

FloatArrayUniform::
~FloatArrayUniform() = default;

FloatArray
FloatArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatArrayUniformImpl>(pimpl())->get();
}

void
FloatArrayUniform::
set(const FloatArray& v)
{
    dynamic_pointer_cast<internal::FloatArrayUniformImpl>(pimpl())->set(v);
}

FloatArrayUniform::
operator FloatArray() const
{
    return get();
}

FloatArrayUniform&
FloatArrayUniform::
operator=(const FloatArray& rhs)
{
    set(rhs);
    return *this;
}

FloatArrayUniform&
FloatArrayUniform::
operator=(const FloatArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatVec2Array

FloatVec2ArrayUniform::
FloatVec2ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatVec2ArrayUniformImpl>(prog_id, location, name, size))
{
}

FloatVec2ArrayUniform::
~FloatVec2ArrayUniform() = default;

FloatVec2Array
FloatVec2ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatVec2ArrayUniformImpl>(pimpl())->get();
}

void
FloatVec2ArrayUniform::
set(const FloatVec2Array& v)
{
    dynamic_pointer_cast<internal::FloatVec2ArrayUniformImpl>(pimpl())->set(v);
}

FloatVec2ArrayUniform::
operator FloatVec2Array() const
{
    return get();
}

FloatVec2ArrayUniform&
FloatVec2ArrayUniform::
operator=(const FloatVec2Array& rhs)
{
    set(rhs);
    return *this;
}

FloatVec2ArrayUniform&
FloatVec2ArrayUniform::
operator=(const FloatVec2ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatVec3Array

FloatVec3ArrayUniform::
FloatVec3ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatVec3ArrayUniformImpl>(prog_id, location, name, size))
{
}

FloatVec3ArrayUniform::
~FloatVec3ArrayUniform() = default;

FloatVec3Array
FloatVec3ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatVec3ArrayUniformImpl>(pimpl())->get();
}

void
FloatVec3ArrayUniform::
set(const FloatVec3Array& v)
{
    dynamic_pointer_cast<internal::FloatVec3ArrayUniformImpl>(pimpl())->set(v);
}

FloatVec3ArrayUniform::
operator FloatVec3Array() const
{
    return get();
}

FloatVec3ArrayUniform&
FloatVec3ArrayUniform::
operator=(const FloatVec3Array& rhs)
{
    set(rhs);
    return *this;
}

FloatVec3ArrayUniform&
FloatVec3ArrayUniform::
operator=(const FloatVec3ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatVec4Array

FloatVec4ArrayUniform::
FloatVec4ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatVec4ArrayUniformImpl>(prog_id, location, name, size))
{
}

FloatVec4ArrayUniform::
~FloatVec4ArrayUniform() = default;

FloatVec4Array
FloatVec4ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatVec4ArrayUniformImpl>(pimpl())->get();
}

void
FloatVec4ArrayUniform::
set(const FloatVec4Array& v)
{
    dynamic_pointer_cast<internal::FloatVec4ArrayUniformImpl>(pimpl())->set(v);
}

FloatVec4ArrayUniform::
operator FloatVec4Array() const
{
    return get();
}

FloatVec4ArrayUniform&
FloatVec4ArrayUniform::
operator=(const FloatVec4Array& rhs)
{
    set(rhs);
    return *this;
}

FloatVec4ArrayUniform&
FloatVec4ArrayUniform::
operator=(const FloatVec4ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatMat2Array

FloatMat2ArrayUniform::
FloatMat2ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatMat2ArrayUniformImpl>(prog_id, location, name, size))
{
}

FloatMat2ArrayUniform::
~FloatMat2ArrayUniform() = default;

FloatMat2Array
FloatMat2ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatMat2ArrayUniformImpl>(pimpl())->get();
}

void
FloatMat2ArrayUniform::
set(const FloatMat2Array& v)
{
    dynamic_pointer_cast<internal::FloatMat2ArrayUniformImpl>(pimpl())->set(v);
}

FloatMat2ArrayUniform::
operator FloatMat2Array() const
{
    return get();
}

FloatMat2ArrayUniform&
FloatMat2ArrayUniform::
operator=(const FloatMat2Array& rhs)
{
    set(rhs);
    return *this;
}

FloatMat2ArrayUniform&
FloatMat2ArrayUniform::
operator=(const FloatMat2ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatMat3Array

FloatMat3ArrayUniform::
FloatMat3ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatMat3ArrayUniformImpl>(prog_id, location, name, size))
{
}

FloatMat3ArrayUniform::
~FloatMat3ArrayUniform() = default;

FloatMat3Array
FloatMat3ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatMat3ArrayUniformImpl>(pimpl())->get();
}

void
FloatMat3ArrayUniform::
set(const FloatMat3Array& v)
{
    dynamic_pointer_cast<internal::FloatMat3ArrayUniformImpl>(pimpl())->set(v);
}

FloatMat3ArrayUniform::
operator FloatMat3Array() const
{
    return get();
}

FloatMat3ArrayUniform&
FloatMat3ArrayUniform::
operator=(const FloatMat3Array& rhs)
{
    set(rhs);
    return *this;
}

FloatMat3ArrayUniform&
FloatMat3ArrayUniform::
operator=(const FloatMat3ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// FloatMat4Array

FloatMat4ArrayUniform::
FloatMat4ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::FloatMat4ArrayUniformImpl>(prog_id, location, name, size))
{
}

FloatMat4ArrayUniform::
~FloatMat4ArrayUniform() = default;

FloatMat4Array
FloatMat4ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::FloatMat4ArrayUniformImpl>(pimpl())->get();
}

void
FloatMat4ArrayUniform::
set(const FloatMat4Array& v)
{
    dynamic_pointer_cast<internal::FloatMat4ArrayUniformImpl>(pimpl())->set(v);
}

FloatMat4ArrayUniform::
operator FloatMat4Array() const
{
    return get();
}

FloatMat4ArrayUniform&
FloatMat4ArrayUniform::
operator=(const FloatMat4Array& rhs)
{
    set(rhs);
    return *this;
}

FloatMat4ArrayUniform&
FloatMat4ArrayUniform::
operator=(const FloatMat4ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// Int

IntUniform::
IntUniform(GLuint prog_id, GLint location, const std::string& name, GLint size)
    : IntUniformVariableBase(
        make_shared<internal::IntUniformImpl>(prog_id, location, name, size))
{
}

IntUniform::
~IntUniform() = default;

IntUniform&
IntUniform::
operator=(GLint rhs)
{
    set(rhs);
    return *this;
}

IntUniform&
IntUniform::
operator=(const IntUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// IntVec2

IntVec2Uniform::
IntVec2Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::IntVec2UniformImpl>(prog_id, location, name, size))
{
}

IntVec2Uniform::
~IntVec2Uniform() = default;

IntVec2
IntVec2Uniform::
get() const
{
    return dynamic_pointer_cast<internal::IntVec2UniformImpl>(pimpl())->get();
}

void
IntVec2Uniform::
get(GLint& x, GLint& y) const
{
    IntVec2 v = dynamic_pointer_cast<internal::IntVec2UniformImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
}

void
IntVec2Uniform::
set(const IntVec2& v)
{
    dynamic_pointer_cast<internal::IntVec2UniformImpl>(pimpl())->set(v);
}

void
IntVec2Uniform::
set(GLint x, GLint y)
{
    IntVec2 v = { { x, y } };
    dynamic_pointer_cast<internal::IntVec2UniformImpl>(pimpl())->set(v);
}

IntVec2Uniform::
operator IntVec2() const
{
    return get();
}

IntVec2Uniform&
IntVec2Uniform::
operator=(const IntVec2& rhs)
{
    set(rhs);
    return *this;
}

IntVec2Uniform&
IntVec2Uniform::
operator=(const IntVec2Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// IntVec3

IntVec3Uniform::
IntVec3Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::IntVec3UniformImpl>(prog_id, location, name, size))
{
}

IntVec3Uniform::
~IntVec3Uniform() = default;

IntVec3
IntVec3Uniform::
get() const
{
    return dynamic_pointer_cast<internal::IntVec3UniformImpl>(pimpl())->get();
}

void
IntVec3Uniform::
get(GLint& x, GLint& y, GLint& z) const
{
    IntVec3 v = dynamic_pointer_cast<internal::IntVec3UniformImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
    z = v[2];
}

void
IntVec3Uniform::
set(const IntVec3& v)
{
    dynamic_pointer_cast<internal::IntVec3UniformImpl>(pimpl())->set(v);
}

void
IntVec3Uniform::
set(GLint x, GLint y, GLint z)
{
    IntVec3 v = { { x, y, z } };
    dynamic_pointer_cast<internal::IntVec3UniformImpl>(pimpl())->set(v);
}

IntVec3Uniform::
operator IntVec3() const
{
    return get();
}

IntVec3Uniform&
IntVec3Uniform::
operator=(const IntVec3& rhs)
{
    set(rhs);
    return *this;
}

IntVec3Uniform&
IntVec3Uniform::
operator=(const IntVec3Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// IntVec4

IntVec4Uniform::
IntVec4Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::IntVec4UniformImpl>(prog_id, location, name, size))
{
}

IntVec4Uniform::
~IntVec4Uniform() = default;

IntVec4
IntVec4Uniform::
get() const
{
    return dynamic_pointer_cast<internal::IntVec4UniformImpl>(pimpl())->get();
}

void
IntVec4Uniform::
get(GLint& x, GLint& y, GLint& z, GLint& w) const
{
    IntVec4 v = dynamic_pointer_cast<internal::IntVec4UniformImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
    z = v[2];
    w = v[3];
}

void
IntVec4Uniform::
set(const IntVec4& v)
{
    dynamic_pointer_cast<internal::IntVec4UniformImpl>(pimpl())->set(v);
}

void
IntVec4Uniform::
set(GLint x, GLint y, GLint z, GLint w)
{
    IntVec4 v = { { x, y, z, w } };
    dynamic_pointer_cast<internal::IntVec4UniformImpl>(pimpl())->set(v);
}

IntVec4Uniform::
operator IntVec4() const
{
    return get();
}

IntVec4Uniform&
IntVec4Uniform::
operator=(const IntVec4& rhs)
{
    set(rhs);
    return *this;
}

IntVec4Uniform&
IntVec4Uniform::
operator=(const IntVec4Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// IntArray

IntArrayUniform::
IntArrayUniform(GLuint prog_id, GLint location, const std::string& name, GLint size)
    : IntArrayUniformVariableBase(
        make_shared<internal::IntArrayUniformImpl>(prog_id, location, name, size))
{
}

IntArrayUniform::
~IntArrayUniform() = default;

IntArrayUniform&
IntArrayUniform::
operator=(const IntArray& rhs)
{
    set(rhs);
    return *this;
}

IntArrayUniform&
IntArrayUniform::
operator=(const IntArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// IntVec2Array

IntVec2ArrayUniform::
IntVec2ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::IntVec2ArrayUniformImpl>(prog_id, location, name, size))
{
}

IntVec2ArrayUniform::
~IntVec2ArrayUniform() = default;

IntVec2Array
IntVec2ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::IntVec2ArrayUniformImpl>(pimpl())->get();
}

void
IntVec2ArrayUniform::
set(const IntVec2Array& v)
{
    dynamic_pointer_cast<internal::IntVec2ArrayUniformImpl>(pimpl())->set(v);
}

IntVec2ArrayUniform::
operator IntVec2Array() const
{
    return get();
}

IntVec2ArrayUniform&
IntVec2ArrayUniform::
operator=(const IntVec2Array& rhs)
{
    set(rhs);
    return *this;
}

IntVec2ArrayUniform&
IntVec2ArrayUniform::
operator=(const IntVec2ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// IntVec3Array

IntVec3ArrayUniform::
IntVec3ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::IntVec3ArrayUniformImpl>(prog_id, location, name, size))
{
}

IntVec3ArrayUniform::
~IntVec3ArrayUniform() = default;

IntVec3Array
IntVec3ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::IntVec3ArrayUniformImpl>(pimpl())->get();
}

void
IntVec3ArrayUniform::
set(const IntVec3Array& v)
{
    dynamic_pointer_cast<internal::IntVec3ArrayUniformImpl>(pimpl())->set(v);
}

IntVec3ArrayUniform::
operator IntVec3Array() const
{
    return get();
}

IntVec3ArrayUniform&
IntVec3ArrayUniform::
operator=(const IntVec3Array& rhs)
{
    set(rhs);
    return *this;
}

IntVec3ArrayUniform&
IntVec3ArrayUniform::
operator=(const IntVec3ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// IntVec4Array

IntVec4ArrayUniform::
IntVec4ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::IntVec4ArrayUniformImpl>(prog_id, location, name, size))
{
}

IntVec4ArrayUniform::
~IntVec4ArrayUniform() = default;

IntVec4Array
IntVec4ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::IntVec4ArrayUniformImpl>(pimpl())->get();
}

void
IntVec4ArrayUniform::
set(const IntVec4Array& v)
{
    dynamic_pointer_cast<internal::IntVec4ArrayUniformImpl>(pimpl())->set(v);
}

IntVec4ArrayUniform::
operator IntVec4Array() const
{
    return get();
}

IntVec4ArrayUniform&
IntVec4ArrayUniform::
operator=(const IntVec4Array& rhs)
{
    set(rhs);
    return *this;
}

IntVec4ArrayUniform&
IntVec4ArrayUniform::
operator=(const IntVec4ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// Bool

BoolUniform::
BoolUniform(GLuint prog_id, GLint location, const std::string& name, GLint size)
    : UniformVariable(make_shared<internal::BoolUniformImpl>(prog_id, location, name, size))
{
}

BoolUniform::
~BoolUniform() = default;

GLboolean
BoolUniform::
get() const
{
    return dynamic_pointer_cast<internal::BoolUniformImpl>(pimpl())->get();
}

void
BoolUniform::
set(GLboolean v)
{
    dynamic_pointer_cast<internal::BoolUniformImpl>(pimpl())->set(v);
}

BoolUniform::
operator GLboolean() const
{
    return get();
}

BoolUniform&
BoolUniform::
operator=(GLboolean rhs)
{
    set(rhs);
    return *this;
}

BoolUniform&
BoolUniform::
operator=(const BoolUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// BoolVec2

BoolVec2Uniform::
BoolVec2Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::BoolVec2UniformImpl>(prog_id, location, name, size))
{
}

BoolVec2Uniform::
~BoolVec2Uniform() = default;

BoolVec2
BoolVec2Uniform::
get() const
{
    return dynamic_pointer_cast<internal::BoolVec2UniformImpl>(pimpl())->get();
}

void
BoolVec2Uniform::
get(GLboolean& x, GLboolean& y) const
{
    BoolVec2 v = dynamic_pointer_cast<internal::BoolVec2UniformImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
}

void
BoolVec2Uniform::
set(const BoolVec2& v)
{
    dynamic_pointer_cast<internal::BoolVec2UniformImpl>(pimpl())->set(v);
}

void
BoolVec2Uniform::
set(GLboolean x, GLboolean y)
{
    BoolVec2 v = { { x, y } };
    dynamic_pointer_cast<internal::BoolVec2UniformImpl>(pimpl())->set(v);
}

BoolVec2Uniform::
operator BoolVec2() const
{
    return get();
}

BoolVec2Uniform&
BoolVec2Uniform::
operator=(const BoolVec2& rhs)
{
    set(rhs);
    return *this;
}

BoolVec2Uniform&
BoolVec2Uniform::
operator=(const BoolVec2Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// BoolVec3

BoolVec3Uniform::
BoolVec3Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::BoolVec3UniformImpl>(prog_id, location, name, size))
{
}

BoolVec3Uniform::
~BoolVec3Uniform() = default;

BoolVec3
BoolVec3Uniform::
get() const
{
    return dynamic_pointer_cast<internal::BoolVec3UniformImpl>(pimpl())->get();
}

void
BoolVec3Uniform::
get(GLboolean& x, GLboolean& y, GLboolean& z) const
{
    BoolVec3 v = dynamic_pointer_cast<internal::BoolVec3UniformImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
    z = v[2];
}

void
BoolVec3Uniform::
set(const BoolVec3& v)
{
    dynamic_pointer_cast<internal::BoolVec3UniformImpl>(pimpl())->set(v);
}

void
BoolVec3Uniform::
set(GLboolean x, GLboolean y, GLboolean z)
{
    BoolVec3 v = { { x, y, z } };
    dynamic_pointer_cast<internal::BoolVec3UniformImpl>(pimpl())->set(v);
}

BoolVec3Uniform::
operator BoolVec3() const
{
    return get();
}

BoolVec3Uniform&
BoolVec3Uniform::
operator=(const BoolVec3& rhs)
{
    set(rhs);
    return *this;
}

BoolVec3Uniform&
BoolVec3Uniform::
operator=(const BoolVec3Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// BoolVec4

BoolVec4Uniform::
BoolVec4Uniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::BoolVec4UniformImpl>(prog_id, location, name, size))
{
}

BoolVec4Uniform::
~BoolVec4Uniform() = default;

BoolVec4
BoolVec4Uniform::
get() const
{
    return dynamic_pointer_cast<internal::BoolVec4UniformImpl>(pimpl())->get();
}

void
BoolVec4Uniform::
get(GLboolean& x, GLboolean& y, GLboolean& z, GLboolean& w) const
{
    BoolVec4 v = dynamic_pointer_cast<internal::BoolVec4UniformImpl>(pimpl())->get();
    x = v[0];
    y = v[1];
    z = v[2];
    w = v[3];
}

void
BoolVec4Uniform::
set(const BoolVec4& v)
{
    dynamic_pointer_cast<internal::BoolVec4UniformImpl>(pimpl())->set(v);
}

void
BoolVec4Uniform::
set(GLboolean x, GLboolean y, GLboolean z, GLboolean w)
{
    BoolVec4 v = { { x, y, z, w } };
    dynamic_pointer_cast<internal::BoolVec4UniformImpl>(pimpl())->set(v);
}

BoolVec4Uniform::
operator BoolVec4() const
{
    return get();
}

BoolVec4Uniform&
BoolVec4Uniform::
operator=(const BoolVec4& rhs)
{
    set(rhs);
    return *this;
}

BoolVec4Uniform&
BoolVec4Uniform::
operator=(const BoolVec4Uniform& rhs)
{
    set(rhs.get());
    return *this;
}

// BoolArray

BoolArrayUniform::
BoolArrayUniform(GLuint prog_id, GLint location, const std::string& name, GLint size)
    : UniformVariable(make_shared<internal::BoolArrayUniformImpl>(prog_id, location, name, size))
{
}

BoolArrayUniform::
~BoolArrayUniform() = default;

BoolArray
BoolArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::BoolArrayUniformImpl>(pimpl())->get();
}

void
BoolArrayUniform::
set(const BoolArray& v)
{
    dynamic_pointer_cast<internal::BoolArrayUniformImpl>(pimpl())->set(v);
}

BoolArrayUniform::
operator BoolArray() const
{
    return get();
}

BoolArrayUniform&
BoolArrayUniform::
operator=(const BoolArray& rhs)
{
    set(rhs);
    return *this;
}

BoolArrayUniform&
BoolArrayUniform::
operator=(const BoolArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// BoolVec2Array

BoolVec2ArrayUniform::
BoolVec2ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::BoolVec2ArrayUniformImpl>(prog_id, location, name, size))
{
}

BoolVec2ArrayUniform::
~BoolVec2ArrayUniform() = default;

BoolVec2Array
BoolVec2ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::BoolVec2ArrayUniformImpl>(pimpl())->get();
}

void
BoolVec2ArrayUniform::
set(const BoolVec2Array& v)
{
    dynamic_pointer_cast<internal::BoolVec2ArrayUniformImpl>(pimpl())->set(v);
}

BoolVec2ArrayUniform::
operator BoolVec2Array() const
{
    return get();
}

BoolVec2ArrayUniform&
BoolVec2ArrayUniform::
operator=(const BoolVec2Array& rhs)
{
    set(rhs);
    return *this;
}

BoolVec2ArrayUniform&
BoolVec2ArrayUniform::
operator=(const BoolVec2ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// BoolVec3Array

BoolVec3ArrayUniform::
BoolVec3ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::BoolVec3ArrayUniformImpl>(prog_id, location, name, size))
{
}

BoolVec3ArrayUniform::
~BoolVec3ArrayUniform() = default;

BoolVec3Array
BoolVec3ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::BoolVec3ArrayUniformImpl>(pimpl())->get();
}

void
BoolVec3ArrayUniform::
set(const BoolVec3Array& v)
{
    dynamic_pointer_cast<internal::BoolVec3ArrayUniformImpl>(pimpl())->set(v);
}

BoolVec3ArrayUniform::
operator BoolVec3Array() const
{
    return get();
}

BoolVec3ArrayUniform&
BoolVec3ArrayUniform::
operator=(const BoolVec3Array& rhs)
{
    set(rhs);
    return *this;
}

BoolVec3ArrayUniform&
BoolVec3ArrayUniform::
operator=(const BoolVec3ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// BoolVec4Array

BoolVec4ArrayUniform::
BoolVec4ArrayUniform(GLuint prog_id, GLint location, const string& name, GLint size)
    : UniformVariable(make_shared<internal::BoolVec4ArrayUniformImpl>(prog_id, location, name, size))
{
}

BoolVec4ArrayUniform::
~BoolVec4ArrayUniform() = default;

BoolVec4Array
BoolVec4ArrayUniform::
get() const
{
    return dynamic_pointer_cast<internal::BoolVec4ArrayUniformImpl>(pimpl())->get();
}

void
BoolVec4ArrayUniform::
set(const BoolVec4Array& v)
{
    dynamic_pointer_cast<internal::BoolVec4ArrayUniformImpl>(pimpl())->set(v);
}

BoolVec4ArrayUniform::
operator BoolVec4Array() const
{
    return get();
}

BoolVec4ArrayUniform&
BoolVec4ArrayUniform::
operator=(const BoolVec4Array& rhs)
{
    set(rhs);
    return *this;
}

BoolVec4ArrayUniform&
BoolVec4ArrayUniform::
operator=(const BoolVec4ArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// Sampler2D

Sampler2DUniform::
Sampler2DUniform(GLuint prog_id, GLint location, const std::string& name, GLint size)
    : IntUniformVariableBase(make_shared<internal::Sampler2DUniformImpl>(prog_id, location, name, size))
{
}

Sampler2DUniform::
~Sampler2DUniform() = default;

Sampler2DUniform&
Sampler2DUniform::
operator=(GLint rhs)
{
    set(rhs);
    return *this;
}

Sampler2DUniform&
Sampler2DUniform::
operator=(const Sampler2DUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// Sampler2DArray

Sampler2DArrayUniform::
Sampler2DArrayUniform(GLuint prog_id, GLint location, const std::string& name, GLint size)
    : IntArrayUniformVariableBase(
        make_shared<internal::Sampler2DArrayUniformImpl>(prog_id, location, name, size))
{
}

Sampler2DArrayUniform::
~Sampler2DArrayUniform() = default;

Sampler2DArrayUniform&
Sampler2DArrayUniform::
operator=(const Sampler2DArray& rhs)
{
    set(rhs);
    return *this;
}

Sampler2DArrayUniform&
Sampler2DArrayUniform::
operator=(const Sampler2DArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// SamplerCube

SamplerCubeUniform::
SamplerCubeUniform(GLuint prog_id, GLint location, const std::string& name, GLint size)
    : IntUniformVariableBase(
        make_shared<internal::SamplerCubeUniformImpl>(prog_id, location, name, size))
{
}

SamplerCubeUniform::
~SamplerCubeUniform() = default;

SamplerCubeUniform&
SamplerCubeUniform::
operator=(GLint rhs)
{
    set(rhs);
    return *this;
}

SamplerCubeUniform&
SamplerCubeUniform::
operator=(const SamplerCubeUniform& rhs)
{
    set(rhs.get());
    return *this;
}

// SamplerCubeArray

SamplerCubeArrayUniform::
SamplerCubeArrayUniform(GLuint prog_id, GLint location, const std::string& name, GLint size)
    : IntArrayUniformVariableBase(
        make_shared<internal::SamplerCubeArrayUniformImpl>(prog_id, location, name, size))
{
}

SamplerCubeArrayUniform::
~SamplerCubeArrayUniform() = default;

SamplerCubeArrayUniform&
SamplerCubeArrayUniform::
operator=(const SamplerCubeArray& rhs)
{
    set(rhs);
    return *this;
}

SamplerCubeArrayUniform&
SamplerCubeArrayUniform::
operator=(const SamplerCubeArrayUniform& rhs)
{
    set(rhs.get());
    return *this;
}

} // namespace glint
