/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/Exception.h>
#include <glint/genv/GEnvException.h>
#include <glint/genv/egl/internal/EGLExceptionImpl.h>

using namespace std;

namespace glint
{

namespace genv
{

namespace egl
{

EGLException::
EGLException(const string& reason, int egl_error)
    : GEnvException(make_shared<glint::genv::egl::internal::EGLExceptionImpl>(reason, egl_error))
{
}

EGLException::
EGLException(const EGLException&) = default;

EGLException&
EGLException::
operator=(const EGLException&) = default;

EGLException::
~EGLException() noexcept = default;

const char*
EGLException::
what() const noexcept
{
    return dynamic_pointer_cast<internal::EGLExceptionImpl>(pimpl())->what();
}

exception_ptr
EGLException::
self() const
{
    return make_exception_ptr(*this);
}

string
EGLException::
reason() const
{
    return dynamic_pointer_cast<internal::EGLExceptionImpl>(pimpl())->reason();
}

EGLint
EGLException::
egl_error() const noexcept
{
    return dynamic_pointer_cast<internal::EGLExceptionImpl>(pimpl())->egl_error();
}


} // namespace egl

} // namespace genv

} // namesapce glint
