/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/genv/egl/internal/EGLExceptionImpl.h>

#include <sstream>
#include <iomanip>

using namespace std;

namespace glint
{

namespace genv
{

namespace egl
{

namespace internal
{

EGLExceptionImpl::
EGLExceptionImpl(const string& reason, EGLint egl_error)
    : GEnvExceptionImpl(reason)
    , egl_error_(egl_error)
{
}

const char*
EGLExceptionImpl::
what() const noexcept
{
    return "glint::genv::egl::EGLException";
}

string
EGLExceptionImpl::
reason() const
{
    ostringstream r;
    r << glint::internal::ExceptionImpl::reason() << ": egl_error = 0x" << hex << setfill('0') << setw(4) << egl_error_
      << " (" << egl_error_string() << ")";
    return r.str();
}

string
EGLExceptionImpl::
egl_error_string() const
{
    switch (egl_error_)
    {
        case EGL_SUCCESS:
            return "EGL_SUCCESS: No error";
        case EGL_NOT_INITIALIZED:
            return "EGL_NOT_INITIALIZED: EGL is not or could not be initialized for the specified display.";
        case EGL_BAD_ACCESS:
            return "EGL_BAD_ACCESS: EGL cannot access a requested resource, such as a context bound in another thread.";
        case EGL_BAD_ALLOC:
            return "EGL_BAD_ALLOC: EGL failed to allocate resources for the requested operation.";
        case EGL_BAD_ATTRIBUTE:
            return "EGL_BAD_ATTRIBUTE: An unrecognized attribute or attribute value was passed in an attribute list.";
        case EGL_BAD_CONFIG:
            return "EGL_BAD_CONFIG: An EGLConfig argument does not name a valid EGLConfig.";
        case EGL_BAD_CONTEXT:
            return "EGL_BAD_CONTEXT: An EGLContext argument does not name a valid EGLContext.";
        case EGL_BAD_CURRENT_SURFACE:
            return "EGL_BAD_CURRENT_SURFACE: The current surface of the calling thread is a window,"
                   " pbuffer, or pixmap that is no longer valid.";
        case EGL_BAD_DISPLAY:
            return "EGL_BAD_DISPLAY: An EGLDisplay argument does not name a valid EGLDisplay.";
        case EGL_BAD_MATCH:
            return "EGL_BAD_MATCH: Arguments are inconsistent; for example, an otherwise valid context requires buffers"
                   " (e.g. depth or stencil) not allocated by an otherwise valid surface.";
        case EGL_BAD_NATIVE_PIXMAP:
            return "EGL_BAD_NATIVE_PIXMAP: An EGLNativePixmapType argument does not refer to a valid native pixmap.";
        case EGL_BAD_NATIVE_WINDOW:
            return "EGL_BAD_NATIVE_WINDOW: An EGLNativeWindowType argument does not refer to a valid native window.";
        case EGL_BAD_PARAMETER:
            return "EGL_BAD_PARAMETER: One or more argument values are invalid.";
        case EGL_BAD_SURFACE:
            return "EGL_BAD_SURFACE: An EGLSurface argument does not name a valid surface (window, pbuffer, or pixmap)"
                   " configured for rendering.";
        case EGL_CONTEXT_LOST:
            return "EGL_CONTEXT_LOST: A power management event has occurred. This application must destroy all contexts"
                   " and reinitialize client API state and objects to continue rendering.";
        default:
            return "Unknown EGL error code";
    }
}

EGLint
EGLExceptionImpl::
egl_error() const noexcept
{
    return egl_error_;
}

} // namespace internal

} // namespace egl

} // namespace genv

} // namespace glint
