/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/genv/egl/internal/EGLGraphicsDisplay.h>

using namespace std;

namespace glint
{

namespace genv
{

namespace egl
{

namespace internal
{

EGLGraphicsDisplay::
EGLGraphicsDisplay(genv::internal::AbstractGEnv& genv, genv::internal::AbstractWEnv& wenv)
    : genv_(genv)
    , wenv_(wenv)
{
}

EGLGraphicsDisplay::
~EGLGraphicsDisplay() noexcept
{
}

int
EGLGraphicsDisplay::
gl_major_version() const noexcept
{
    return genv_.gl_major_version();
}

int
EGLGraphicsDisplay::
gl_minor_version() const noexcept
{
    return genv_.gl_minor_version();
}

int
EGLGraphicsDisplay::
width() const noexcept
{
    return wenv_.width();
}

int
EGLGraphicsDisplay::
height() const noexcept
{
    return wenv_.height();
}

int
EGLGraphicsDisplay::
xpos() const noexcept
{
    return wenv_.xpos();
}

int
EGLGraphicsDisplay::
ypos() const noexcept
{
    return wenv_.ypos();
}

void
EGLGraphicsDisplay::
make_current()
{
    return genv_.make_current();
}

void
EGLGraphicsDisplay::
release_current()
{
    return genv_.release_current();
}

void
EGLGraphicsDisplay::
swap_buffers()
{
    return genv_.swap_buffers();
}

GraphicsDisplay::Vis
EGLGraphicsDisplay::
set_visibility(GraphicsDisplay::Vis v)
{
    using namespace genv::internal;

    AbstractWEnv::Vis vis = v == GraphicsDisplay::Vis::visible ? AbstractWEnv::Vis::visible : AbstractWEnv::Vis::invisible;
    AbstractWEnv::Vis old_vis = wenv_.set_visibility(vis);
    return old_vis == AbstractWEnv::Vis::visible ? GraphicsDisplay::Vis::visible : GraphicsDisplay::Vis::invisible;
}

GraphicsDisplay::Vis
EGLGraphicsDisplay::
visibility()
{
    using namespace genv::internal;

    AbstractWEnv::Vis vis = wenv_.visibility();
    return vis == AbstractWEnv::Vis::visible ? GraphicsDisplay::Vis::visible : GraphicsDisplay::Vis::invisible;
}

void
EGLGraphicsDisplay::
set_stacking_order(GraphicsDisplay::StackingOrder order)
{
    using namespace genv::internal;

    AbstractWEnv::StackingOrder o = order == GraphicsDisplay::StackingOrder::top
                                          ? AbstractWEnv::StackingOrder::top
                                          : AbstractWEnv::StackingOrder::bottom;
    wenv_.set_stacking_order(o);
}

AbstractWindow*
EGLGraphicsDisplay::
window() const noexcept
{
    return wenv_.window();
}

} // namespace internal

} // namespace egl

} // namespace genv

} // namespace glint
