/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/genv/glx/internal/GLXGraphicsDisplay.h>
#include <glint/genv/glx/GLXException.h>

using namespace glint::internal;
using namespace std;

namespace glint
{

namespace genv
{

namespace glx
{

namespace internal
{

GLXGraphicsDisplay::
GLXGraphicsDisplay(genv::internal::AbstractGEnv& genv, genv::internal::AbstractWEnv& wenv)
    : genv_(genv)
    , wenv_(wenv)
{
}

GLXGraphicsDisplay::
~GLXGraphicsDisplay() noexcept
{
}

int
GLXGraphicsDisplay::
gl_major_version() const noexcept
{
    return genv_.gl_major_version();
}

int
GLXGraphicsDisplay::
gl_minor_version() const noexcept
{
    return genv_.gl_minor_version();
}

int
GLXGraphicsDisplay::
width() const noexcept
{
    return wenv_.width();
}

int
GLXGraphicsDisplay::
height() const noexcept
{
    return wenv_.height();
}

int
GLXGraphicsDisplay::
xpos() const noexcept
{
    return wenv_.xpos();
}

int
GLXGraphicsDisplay::
ypos() const noexcept
{
    return wenv_.ypos();
}

void
GLXGraphicsDisplay::
make_current()
{
    genv_.make_current();
}

void
GLXGraphicsDisplay::
release_current()
{
    genv_.release_current();
}

void
GLXGraphicsDisplay::
swap_buffers()
{
    genv_.swap_buffers();
}

GraphicsDisplay::Vis
GLXGraphicsDisplay::
set_visibility(GraphicsDisplay::Vis v)
{
    using namespace genv::internal;

    AbstractWEnv::Vis vis = v == GraphicsDisplay::Vis::visible ? AbstractWEnv::Vis::visible : AbstractWEnv::Vis::invisible;
    AbstractWEnv::Vis old_vis = wenv_.set_visibility(vis);
    return old_vis == AbstractWEnv::Vis::visible ? GraphicsDisplay::Vis::visible : GraphicsDisplay::Vis::invisible;
}

GraphicsDisplay::Vis
GLXGraphicsDisplay::
visibility()
{
    using namespace genv::internal;

    AbstractWEnv::Vis vis = wenv_.visibility();
    return vis == AbstractWEnv::Vis::visible ? GraphicsDisplay::Vis::visible : GraphicsDisplay::Vis::invisible;
}

void
GLXGraphicsDisplay::
set_stacking_order(GraphicsDisplay::StackingOrder order)
{
    using namespace genv::internal;

    AbstractWEnv::StackingOrder o = order == GraphicsDisplay::StackingOrder::top
                                          ? AbstractWEnv::StackingOrder::top
                                          : AbstractWEnv::StackingOrder::bottom;
    wenv_.set_stacking_order(o);
}

AbstractWindow*
GLXGraphicsDisplay::
window() const noexcept
{
    return wenv_.window();
}

} // namespace internal

} // namespace glx

} // namespace genv

} // namespace glint
