/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/internal/GlintExceptionsImpl.h>
#include <glint/GLUtil.h>

using namespace std;

namespace glint
{

namespace internal
{

OpenGLExceptionImpl::
OpenGLExceptionImpl(const string& reason, int gl_error)
    : ExceptionImpl(reason + (reason.empty() ? "" : " ") + "(" + gl_err_to_name(gl_error) + ")")
    , gl_error_(gl_error)
{
}

const char*
OpenGLExceptionImpl::
what() const noexcept
{
    return "glint::OpenGLException";
}

int
OpenGLExceptionImpl::
gl_error() const noexcept
{
    return gl_error_;
}

InvalidArgumentExceptionImpl::
InvalidArgumentExceptionImpl(const string& reason)
    : ExceptionImpl(reason)
{
}

const char*
InvalidArgumentExceptionImpl::
what() const noexcept
{
    return "glint::InvalidArgumentException";
}

LogicExceptionImpl::
LogicExceptionImpl(const string& reason)
    : ExceptionImpl(reason)
{
}

const char*
LogicExceptionImpl::
what() const noexcept
{
    return "glint::LogicException";
}

ShutdownExceptionImpl::
ShutdownExceptionImpl(const string& reason)
    : ExceptionImpl(reason)
{
}

const char*
ShutdownExceptionImpl::
what() const noexcept
{
    return "glint::ShutdownException";
}

FileExceptionImpl::
FileExceptionImpl(const string& reason, int err)
    : ExceptionImpl(reason + (err == 0 ? "" : " (errno = " + std::to_string(err) + ")"))
    , errno_(err)
{
}

const char*
FileExceptionImpl::
what() const noexcept
{
    return "glint::FileException";
}

int
FileExceptionImpl::
error() const noexcept
{
    return errno_;
}

} // namespace internal

} // namespace glint
