/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/util/FileIO.h>
#include <glint/util/ResourcePtr.h>
#include <glint/GlintExceptions.h>

#include <fcntl.h>
#include <sys/stat.h>

using namespace std;

namespace glint
{

namespace util
{

namespace
{

//
// It would be nice to use fstream for I/O, but the error reporting is so useless that it's better to step
// down to system calls. At least then, when something goes wrong, we know what it was.
//

vector<char>
read_file(const string& filename)
{
    util::ResourcePtr<int, decltype(&::close)> fd(::open(filename.c_str(), O_RDONLY), ::close);
    if (fd.get() == -1)
    {
        throw FileException("cannot open \"" + filename + "\": " + strerror(errno), errno);
    }

    struct stat st;
    if (fstat(fd.get(), &st) == -1)
    {
        throw FileException("cannot fstat \"" + filename + "\": " + strerror(errno), errno);
    }

    if (!S_ISREG(st.st_mode))
    {
        throw FileException("\"" + filename + "\" is not a regular file", 0);
    }

    if (st.st_size == 0)
    {
        throw FileException("file \"" + filename + "\" is empty", 0);
    }

    vector<char> buf(st.st_size);
    if (read(fd.get(), &buf[0], st.st_size) != st.st_size)
    {
        ostringstream msg;
        msg << "cannot read " << st.st_size << " byte";
        if (st.st_size != 1)
        {
            msg << "s";
        }
        msg << " from \"" << filename << "\": " << strerror(errno);
        throw FileException(msg.str(), errno);
    }

    return buf;
}

} // namespace

string
read_text_file(const string& filename)
{
    vector<char> buf(read_file(filename));
    return string(buf.begin(), buf.end());
}

vector<char>
read_binary_file(const string& filename)
{
    return read_file(filename);
}

} // namespace util

} // namespace glint
