/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/GLUtil.h>
#include <glint/Runtime.h>
#include <glint/GlintExceptions.h>

#include <gtest/gtest.h>

using namespace glint;
using namespace std;

TEST(GLutil, gl_err_to_name)
{
    EXPECT_EQ("GL_NO_ERROR", gl_err_to_name(GL_NO_ERROR));
    EXPECT_EQ("GL_INVALID_ENUM", gl_err_to_name(GL_INVALID_ENUM));
    EXPECT_EQ("GL_INVALID_VALUE", gl_err_to_name(GL_INVALID_VALUE));
    EXPECT_EQ("GL_INVALID_OPERATION", gl_err_to_name(GL_INVALID_OPERATION));
    EXPECT_EQ("GL_OUT_OF_MEMORY", gl_err_to_name(GL_OUT_OF_MEMORY));
    EXPECT_EQ("GL_INVALID_FRAMEBUFFER_OPERATION", gl_err_to_name(GL_INVALID_FRAMEBUFFER_OPERATION));
    GLenum e = 1 + max(GL_NO_ERROR,
                       max(GL_INVALID_ENUM,
                           max(GL_INVALID_VALUE,
                               max(GL_INVALID_OPERATION,
                                   max(GL_OUT_OF_MEMORY, GL_INVALID_FRAMEBUFFER_OPERATION)))));
    ostringstream s;
    s << "Unknown GL error: " << e << " (" << hex << "0x" << e << ")";
    EXPECT_EQ(s.str(), gl_err_to_name(e));
}

TEST(GLutil, check_gl)
{
    Runtime::UPtr rt = Runtime::create();
    rt->create_display();

    try
    {
        check_gl("No error");
    }
    catch (...)
    {
        FAIL();
    }

    try
    {
        glClear(0xffffffff);
        check_gl("glClear() failed");
        FAIL();
    }
    catch (const OpenGLException& e)
    {
        EXPECT_EQ("glint::OpenGLException: glClear() failed (GL_INVALID_VALUE)", e.to_string());
    }

    try
    {
        glActiveTexture(GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS);
        check_gl("glGetAttachedShaders() failed");
        FAIL();
    }
    catch (const OpenGLException& e)
    {
        EXPECT_EQ("glint::OpenGLException: glGetAttachedShaders() failed (GL_INVALID_ENUM)", e.to_string());
    }
}
