/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/Point.h>
#include <gtest/gtest.h>

#include <unordered_map>

using namespace glint;

TEST(Point, constructors)
{
    // Default constructor

    EXPECT_EQ(0, Point().x());
    EXPECT_EQ(0, Point().y());
    EXPECT_EQ(0.0f, PointF().x());
    EXPECT_EQ(0.0f, PointF().y());
    EXPECT_EQ(0.0, PointD().x());
    EXPECT_EQ(0.0, PointD().y());
    EXPECT_EQ(0.0L, PointLD().x());
    EXPECT_EQ(0.0L, PointLD().y());

    // Rounding/non-rounding during construction for int and floating-point coordinate types

    EXPECT_EQ(1, Point(1, 0).x());
    EXPECT_EQ(0, Point(1, 0).y());
    EXPECT_EQ(1, Point(1, 0.1).x());
    EXPECT_EQ(0, Point(1, 0.1).y());
    EXPECT_EQ(2, Point(1.6, 0).x());
    EXPECT_EQ(0, Point(1.6, 0).y());
    EXPECT_EQ(2, Point(1.6, 0.1).x());
    EXPECT_EQ(0, Point(1.6, 0.1).y());

    EXPECT_EQ(1.0f, PointF(1, 0).x());
    EXPECT_EQ(0.0f, PointF(1, 0).y());
    EXPECT_EQ(1.0f, PointF(1, 0.1f).x());
    EXPECT_EQ(0.1f, PointF(1, 0.1f).y());
    EXPECT_EQ(1.6f, PointF(1.6f, 0).x());
    EXPECT_EQ(0.0f, PointF(1.6f, 0).y());
    EXPECT_EQ(1.6f, PointF(1.6f, 0.1f).x());
    EXPECT_EQ(0.1f, PointF(1.6f, 0.1f).y());

    EXPECT_EQ(1.0, PointD(1, 0).x());
    EXPECT_EQ(0.0, PointD(1, 0).y());
    EXPECT_EQ(1.0, PointD(1, 0.1).x());
    EXPECT_EQ(0.1, PointD(1, 0.1).y());
    EXPECT_EQ(1.6, PointD(1.6, 0).x());
    EXPECT_EQ(0.0, PointD(1.6, 0).y());
    EXPECT_EQ(1.6, PointD(1.6, 0.1).x());
    EXPECT_EQ(0.1, PointD(1.6, 0.1).y());

    EXPECT_EQ(1.0L, PointLD(1, 0).x());
    EXPECT_EQ(0.0L, PointLD(1, 0).y());
    EXPECT_EQ(1.0L, PointLD(1, 0.1L).x());
    EXPECT_EQ(0.1L, PointLD(1, 0.1L).y());
    EXPECT_EQ(1.6L, PointLD(1.6L, 0).x());
    EXPECT_EQ(0.0L, PointLD(1.6L, 0).y());
    EXPECT_EQ(1.6L, PointLD(1.6L, 0.1L).x());
    EXPECT_EQ(0.1L, PointLD(1.6L, 0.1L).y());

    // Rounding of values is towards nearest integer

    EXPECT_EQ(-2, Point(-1.6f, -0.1f).x());
    EXPECT_EQ(0, Point(-1.6f, -0.1f).y());
    EXPECT_EQ(-2, Point(-1.6, -0.1).x());
    EXPECT_EQ(0, Point(-1.6, -0.1).y());
    EXPECT_EQ(-2, Point(-1.6, -0.1).x());
    EXPECT_EQ(0, Point(-1.6, -0.1L).y());
}

TEST(Point, copy_constructors)
{
    // Copy constructor conversions (using C++ equality comparison)

    EXPECT_EQ(2, Point(Point(1.6, 0.1)).x());
    EXPECT_EQ(0, Point(Point(1.6, 0.1)).y());
    EXPECT_EQ(2, Point(PointF(1.6f, 0.1f)).x());
    EXPECT_EQ(0, Point(PointF(1.6f, 0.1f)).y());
    EXPECT_EQ(2, Point(PointD(1.6, 0.1)).x());
    EXPECT_EQ(0, Point(PointD(1.6, 0.1)).y());
    EXPECT_EQ(2, Point(PointLD(1.6L, 0.1L)).x());
    EXPECT_EQ(0, Point(PointLD(1.6L, 0.1L)).y());

    EXPECT_EQ(2.0f, PointF(Point(2.0, 1.0)).x());
    EXPECT_EQ(1.0f, PointF(Point(2.0, 1.0)).y());
    EXPECT_EQ(2.0f, PointF(PointF(2.0f, 1.0f)).x());
    EXPECT_EQ(1.0f, PointF(PointF(2.0f, 1.0f)).y());
    EXPECT_EQ(2.0f, PointF(PointD(2.0, 1.0)).x());
    EXPECT_EQ(1.0f, PointF(PointD(2.0, 1.0)).y());
    EXPECT_EQ(2.0f, PointF(PointLD(2.0L, 1.0L)).x());
    EXPECT_EQ(1.0f, PointF(PointLD(2.0L, 1.0L)).y());

    EXPECT_EQ(2.0, PointD(Point(2.0, 1.0)).x());
    EXPECT_EQ(1.0, PointD(Point(2.0, 1.0)).y());
    EXPECT_EQ(2.0, PointD(PointF(2.0f, 1.0f)).x());
    EXPECT_EQ(1.0, PointD(PointF(2.0f, 1.0f)).y());
    EXPECT_EQ(2.0, PointD(PointD(2.0, 1.0)).x());
    EXPECT_EQ(1.0, PointD(PointD(2.0, 1.0)).y());
    EXPECT_EQ(2.0, PointD(PointLD(2.0L, 1.0L)).x());
    EXPECT_EQ(1.0, PointD(PointLD(2.0L, 1.0L)).y());

    EXPECT_EQ(2.0L, PointLD(Point(2.0, 1.0)).x());
    EXPECT_EQ(1.0L, PointLD(Point(2.0, 1.0)).y());
    EXPECT_EQ(2.0L, PointLD(PointF(2.0f, 1.0f)).x());
    EXPECT_EQ(1.0L, PointLD(PointF(2.0f, 1.0f)).y());
    EXPECT_EQ(1.0L, PointLD(PointF(2.0f, 1.0f)).y());
    EXPECT_EQ(2.0L, PointLD(PointD(2.0, 1.0)).x());
    EXPECT_EQ(1.0L, PointLD(PointD(2.0, 1.0)).y());
    EXPECT_EQ(2.0L, PointLD(PointLD(2.0L, 1.0L)).x());
    EXPECT_EQ(1.0L, PointLD(PointLD(2.0L, 1.0L)).y());
}

TEST(Point, equal)
{
    // Test that we compare correctly for different values when
    // the first, second, or both coords differ.

    // For different types

    EXPECT_FALSE(PointF(0, 0) == PointD(1, 0));
    EXPECT_FALSE(PointF(0, 0) == PointD(0, 1));
    EXPECT_FALSE(PointF(1, 2) == PointD(3, 4));

    // For int with float comparison

    EXPECT_FALSE(Point(0, 0) == PointD(1, 0));
    EXPECT_FALSE(Point(0, 0) == PointD(0, 1));
    EXPECT_FALSE(Point(1, 2) == PointD(3, 4));

    // For int with int comparison

    EXPECT_FALSE(Point(0, 0) == Point(1, 0));
    EXPECT_FALSE(Point(0, 0) == Point(0, 1));
    EXPECT_FALSE(Point(1, 2) == Point(3, 4));

    // Test that we compare equal for int with float comparison

    EXPECT_TRUE(Point(2, 0) == Point(2, 0));
    EXPECT_TRUE(Point(2, 0) == PointF(2.0f, 0.0f));
    EXPECT_TRUE(Point(2, 0) == PointD(2.0, 0.0));
    EXPECT_TRUE(Point(2, 0) == PointLD(2.0L, 0.0L));

    // Test that we compare correctly for float with other comparison

    EXPECT_TRUE(PointF(2.0f, 0.0f) == Point(2, 0));
    EXPECT_TRUE(PointF(2.0f, 1.0f) == PointF(2.0f, 1.0f));
    EXPECT_TRUE(PointF(2.0f, 1.0f) == PointD(2.0, 1.0));
    EXPECT_TRUE(PointF(2.0f, 1.0f) == PointLD(2.0L, 1.0L));

    // Test that we compare correctly for double with other comparison

    EXPECT_TRUE(PointD(2.0, 0.0) == Point(2, 0));
    EXPECT_TRUE(PointD(2.0, 1.0) == PointF(2.0f, 1.0f));
    EXPECT_TRUE(PointD(2.0, 1.0) == PointD(2.0, 1.0));
    EXPECT_TRUE(PointD(2.0, 1.0) == PointLD(2.0L, 1.0L));

    // Test that we compare correctly for long double with other comparison

    EXPECT_TRUE(PointLD(2.0L, 0.0L) == Point(2, 0));
    EXPECT_TRUE(PointLD(2.0L, 1.0L) == PointF(2.0f, 1.0f));
    EXPECT_TRUE(PointLD(2.0L, 1.0L) == PointD(2.0, 1.0));
    EXPECT_TRUE(PointLD(2.0L, 1.0L) == PointLD(2.0L, 1.0L));
}

TEST(Point, not_equal)
{
    // operator!= is implemented in terms of operator==, so a single test is enough.

    EXPECT_TRUE(Point(2, 0) != Point(1, 0));
}

TEST(Point, assignment)
{
    {
        Point p;
        p = Point(2, 1);
        EXPECT_EQ(2, p.x());
        EXPECT_EQ(1, p.y());

        p = PointF(1.6f, 0.1f);
        EXPECT_EQ(2, p.x());
        EXPECT_EQ(0, p.y());

        p = PointD(1.6, 0.1);
        EXPECT_EQ(2, p.x());
        EXPECT_EQ(0, p.y());

        p = PointLD(1.6L, 0.1L);
        EXPECT_EQ(2, p.x());
        EXPECT_EQ(0, p.y());
    }

    {
        PointF p;
        p = Point(2, 0);
        EXPECT_EQ(2.0f, p.x());
        EXPECT_EQ(0.0f, p.y());

        p = PointF(1.6f, 0.1f);
        EXPECT_EQ(1.6f, p.x());
        EXPECT_EQ(0.1f, p.y());

        p = PointD(1.6, 0.1);
        EXPECT_EQ(1.6f, p.x());
        EXPECT_EQ(0.1f, p.y());

        p = PointLD(1.6L, 0.1L);
        EXPECT_EQ(1.6f, p.x());
        EXPECT_EQ(0.1f, p.y());
    }

    {
        PointD p;
        p = Point(2, 0);
        EXPECT_EQ(2.0, p.x());
        EXPECT_EQ(0.0, p.y());

        p = PointF(1.6f, 0.1f);
        EXPECT_EQ(1.6f, p.x());
        EXPECT_EQ(0.1f, p.y());

        p = PointD(1.6, 0.1);
        EXPECT_EQ(1.6, p.x());
        EXPECT_EQ(0.1, p.y());

        p = PointLD(1.6L, 0.1L);
        EXPECT_EQ(1.6, p.x());
        EXPECT_EQ(0.1, p.y());
    }

    {
        PointLD p;
        p = Point(2, 0);
        EXPECT_EQ(2.0L, p.x());
        EXPECT_EQ(0.0L, p.y());

        p = PointF(1.6f, 0.1f);
        EXPECT_EQ(1.6f, p.x());
        EXPECT_EQ(0.1f, p.y());

        p = PointD(1.6, 0.1);
        EXPECT_EQ(1.6, p.x());
        EXPECT_EQ(0.1, p.y());

        p = PointLD(1.6L, 0.1L);
        EXPECT_EQ(1.6L, p.x());
        EXPECT_EQ(0.1L, p.y());
    }
}

TEST(Point, mutators)
{
    {
        Point p;
        p.set_x(2);
        p.set_y(1);
        EXPECT_EQ(2, p.x());
        EXPECT_EQ(1, p.y());

        p.set_x(1.6f);
        p.set_y(0.1f);
        EXPECT_EQ(2, p.x());
        EXPECT_EQ(0, p.y());

        p.set_x(1.6);
        p.set_y(0.1);
        EXPECT_EQ(2, p.x());
        EXPECT_EQ(0, p.y());

        p.set_x(1.6L);
        p.set_y(0.1L);
        EXPECT_EQ(2, p.x());
        EXPECT_EQ(0, p.y());
    }

    {
        PointF p;
        p.set_x(2);
        p.set_y(0);
        EXPECT_EQ(2.0f, p.x());
        EXPECT_EQ(0.0f, p.y());

        p.set_x(1.6f);
        p.set_y(0.1f);
        EXPECT_EQ(1.6f, p.x());
        EXPECT_EQ(0.1f, p.y());

        p.set_x(1.6);
        p.set_y(0.1);
        EXPECT_EQ(1.6f, p.x()); // Expected, value was narrowed during assignment
        EXPECT_EQ(0.1f, p.y()); // Expected, value was narrowed during assignment

        p.set_x(1.6L);
        p.set_y(0.1L);
        EXPECT_EQ(1.6f, p.x()); // Expected, value was narrowed during assignment
        EXPECT_EQ(0.1f, p.y()); // Expected, value was narrowed during assignment
    }

    {
        PointD p;
        p.set_x(2);
        p.set_y(0);
        EXPECT_EQ(2.0, p.x());
        EXPECT_EQ(0.0, p.y());

        p.set_x(1.6f);
        p.set_y(0.1f);
        EXPECT_EQ(1.6f, p.x()); // Expected loss of precision, value out is only as precise as value in
        EXPECT_EQ(0.1f, p.y()); // Expected loss of precision, value out is only as precise as value in

        p.set_x(1.6);
        p.set_y(0.1);
        EXPECT_EQ(1.6, p.x());
        EXPECT_EQ(0.1, p.y());

        p.set_x(1.6L);
        p.set_y(0.1L);
        EXPECT_EQ(1.6, p.x());  // Expected, value was narrowed during assignment
        EXPECT_EQ(0.1, p.y());  // Expected, value was narrowed during assignment
    }

    {
        PointLD p;
        p.set_x(2);
        p.set_y(0);
        EXPECT_EQ(2.0L, p.x());
        EXPECT_EQ(0.0L, p.y());

        p.set_x(1.6f);
        p.set_y(0.1f);
        EXPECT_EQ(1.6f, p.x()); // Expected loss of precision, value out is only as precise as value in
        EXPECT_EQ(0.1f, p.y()); // Expected loss of precision, value out is only as precise as value in

        p.set_x(1.6);
        p.set_y(0.1);
        EXPECT_EQ(1.6, p.x()); // Expected loss of precision, value out is only as precise as value in
        EXPECT_EQ(0.1, p.y()); // Expected loss of precision, value out is only as precise as value in

        p.set_x(1.6L);
        p.set_y(0.1L);
        EXPECT_EQ(1.6L, p.x());
        EXPECT_EQ(0.1L, p.y());
    }
}

TEST(Point, member_ops)
{
    // is_zero

    EXPECT_TRUE(Point(0, 0).is_zero());
    EXPECT_FALSE(Point(1, 0).is_zero());
    EXPECT_FALSE(Point(0, 1).is_zero());
    EXPECT_FALSE(Point(1, 1).is_zero());

    // operator<

    EXPECT_FALSE(Point(0, 0) < Point(0, 0));
    EXPECT_TRUE(Point(0, 0) < Point(1, 0));
    EXPECT_FALSE(Point(1, 0) < Point(0, 0));
    EXPECT_TRUE(Point(0, 0) < Point(0, 1));
    EXPECT_FALSE(Point(0, 1) < Point(0, 0));

    // operator<=

    EXPECT_TRUE(Point(0, 0) <= Point(0, 0));
    EXPECT_TRUE(Point(0, 0) <= Point(1, 0));
    EXPECT_FALSE(Point(1, 0) <= Point(0, 0));
    EXPECT_TRUE(Point(0, 0) <= Point(0, 1));
    EXPECT_FALSE(Point(0, 1) <= Point(0, 0));

    // operator>

    EXPECT_FALSE(Point(0, 0) > Point(0, 0));
    EXPECT_FALSE(Point(0, 0) > Point(1, 0));
    EXPECT_TRUE(Point(1, 0) > Point(0, 0));
    EXPECT_FALSE(Point(0, 0) > Point(0, 1));
    EXPECT_TRUE(Point(0, 1) > Point(0, 0));

    // operator>=

    EXPECT_TRUE(Point(0, 0) >= Point(0, 0));
    EXPECT_FALSE(Point(0, 0) >= Point(1, 0));
    EXPECT_TRUE(Point(1, 0) >= Point(0, 0));
    EXPECT_FALSE(Point(0, 0) >= Point(0, 1));
    EXPECT_TRUE(Point(0, 1) >= Point(0, 0));

    // operator+

    EXPECT_TRUE(Point(2, 0) == Point(0, 0) + Point(2, 0));
    EXPECT_TRUE(Point(0, 1) == Point(0, 0) + Point(0, 1));
    EXPECT_TRUE(Point(2, 1) == Point(0, 0) + PointF(1.6f, 1.1f));
    EXPECT_TRUE(Point(2, 1) == Point(0, 0) + PointD(1.6, 1.1));
    EXPECT_TRUE(Point(2, 1) == Point(0, 0) + PointLD(1.6L, 1.1L));

    EXPECT_TRUE(PointF(2.0f, 1.0f) == PointF(0, 0) + Point(2, 1));
    EXPECT_TRUE(PointF(1.6f, 1.1f) == PointF(0, 0) + PointF(1.6f, 1.1f));
    EXPECT_TRUE(PointF(1.6f, 1.1f) == PointF(0, 0) + PointD(1.6, 1.1));
    EXPECT_TRUE(PointF(1.6f, 1.1f) == PointF(0, 0) + PointLD(1.6L, 1.1L));

    EXPECT_TRUE(PointD(2.0, 1.0) == PointD(0, 0) + Point(2, 1));
    EXPECT_TRUE(PointD(1.6f, 1.1f) == PointD(0, 0) + PointF(1.6f, 1.1f));
    EXPECT_TRUE(PointD(1.6, 1.1) == PointD(0, 0) + PointD(1.6, 1.1));
    EXPECT_TRUE(PointD(1.6, 1.1) == PointD(0, 0) + PointLD(1.6L, 1.1L));

    EXPECT_TRUE(PointLD(2.0, 1.0) == PointLD(0, 0) + Point(2, 1));
    EXPECT_TRUE(PointLD(1.6f, 1.1f) == PointLD(0, 0) + PointF(1.6f, 1.1f));
    EXPECT_TRUE(PointLD(1.6, 1.1) == PointLD(0, 0) + PointD(1.6, 1.1));
    EXPECT_TRUE(PointLD(1.6L, 1.1L) == PointLD(0, 0) + PointLD(1.6L, 1.1L));

    // operator-

    EXPECT_TRUE(Point(-2, 0) == Point(0, 0) - Point(2, 0));
    EXPECT_TRUE(Point(0, -1) == Point(0, 0) - Point(0, 1));
    EXPECT_TRUE(Point(-2, -1) == Point(0, 0) - PointF(1.6f, 1.1f));
    EXPECT_TRUE(Point(-2, -1) == Point(0, 0) - PointD(1.6, 1.1));
    EXPECT_TRUE(Point(-2, -1) == Point(0, 0) - PointLD(1.6L, 1.1L));

    EXPECT_TRUE(PointF(-2.0f, -1.0f) == PointF(0, 0) - Point(2, 1));
    EXPECT_TRUE(PointF(-1.6f, -1.1f) == PointF(0, 0) - PointF(1.6f, 1.1f));
    EXPECT_TRUE(PointF(-1.6f, -1.1f) == PointF(0, 0) - PointD(1.6, 1.1));
    EXPECT_TRUE(PointF(-1.6f, -1.1f) == PointF(0, 0) - PointLD(1.6L, 1.1L));

    EXPECT_TRUE(PointD(-2.0, -1.0) == PointD(0, 0) - Point(2, 1));
    EXPECT_TRUE(PointD(-1.6f, -1.1f) == PointD(0, 0) - PointF(1.6f, 1.1f));
    EXPECT_TRUE(PointD(-1.6, -1.1) == PointD(0, 0) - PointD(1.6, 1.1));
    EXPECT_TRUE(PointD(-1.6, -1.1) == PointD(0, 0) - PointLD(1.6L, 1.1L));

    EXPECT_TRUE(PointLD(-2.0, -1.0) == PointLD(0, 0) - Point(2, 1));
    EXPECT_TRUE(PointLD(-1.6f, -1.1f) == PointLD(0, 0) - PointF(1.6f, 1.1f));
    EXPECT_TRUE(PointLD(-1.6, -1.1) == PointLD(0, 0) - PointD(1.6, 1.1));
    EXPECT_TRUE(PointLD(-1.6L, -1.1L) == PointLD(0, 0) - PointLD(1.6L, 1.1L));

    // operator*

    EXPECT_TRUE(Point(2, 2) == Point(1, 1) * 2);
    EXPECT_TRUE(Point(2, 2) == Point(1, 1) * 1.6f);
    EXPECT_TRUE(Point(2, 2) == Point(1, 1) * 1.6);
    EXPECT_TRUE(Point(2, 2) == Point(1, 1) * 1.6L);

    EXPECT_TRUE(PointF(2.0f, 2.0f) == PointF(1.0f, 1.0f) * 2);
    EXPECT_TRUE(PointF(1.6f, 1.6f) == PointF(1.0f, 1.0f) * 1.6f);
    EXPECT_TRUE(PointF(1.6f, 1.6f) == PointF(1.0f, 1.0f) * 1.6);
    EXPECT_TRUE(PointF(1.6f, 1.6f) == PointF(1.0f, 1.0f) * 1.6L);

    EXPECT_TRUE(PointD(2.0, 2.0) == PointD(1.0, 1.0) * 2);
    EXPECT_TRUE(PointD(1.6f, 1.6f) == PointD(1.0, 1.0) * 1.6f);     // Expected loss of precision
    EXPECT_TRUE(PointD(1.6, 1.6) == PointD(1.0, 1.0) * 1.6);
    EXPECT_TRUE(PointD(1.6, 1.6) == PointD(1.0, 1.0) * 1.6L);

    EXPECT_TRUE(PointLD(2.0L, 2.0L) == PointLD(1.0L, 1.0L) * 2);
    EXPECT_TRUE(PointLD(1.6f, 1.6f) == PointLD(1.0L, 1.0L) * 1.6f); // Expected loss of precision
    EXPECT_TRUE(PointLD(1.6, 1.6) == PointLD(1.0L, 1.0L) * 1.6);    // Expected loss of precision
    EXPECT_TRUE(PointLD(1.6L, 1.6L) == PointLD(1.0L, 1.0L) * 1.6L);

    // operator/ (int)
    // We use division by 2 here instead of 1.6 because, when diving a double by a float, or a long double
    // by a double or a float, we get rounding issues, where the result is very close to, but not exactly 1.

    EXPECT_TRUE(Point(1, 1) == Point(2, 2) / 2);
    EXPECT_TRUE(Point(1, 1) == Point(2, 2) / 2.0f);
    EXPECT_TRUE(Point(1, 1) == Point(2, 2) / 2.0);
    EXPECT_TRUE(Point(1, 1) == Point(2, 2) / 2.0L);

    EXPECT_TRUE(PointF(1.0f, 1.0f) == PointF(2.0f, 2.0f) / 2);
    EXPECT_TRUE(PointF(1.0f, 1.0f) == PointF(2.0f, 2.0f) / 2.0f);
    EXPECT_TRUE(PointF(1.0f, 1.0f) == PointF(2.0f, 2.0f) / 2.0);
    EXPECT_TRUE(PointF(1.0f, 1.0f) == PointF(2.0f, 2.0f) / 2.0L);

    EXPECT_TRUE(PointD(1.0, 1.0) == PointD(2.0, 2.0) / 2);
    EXPECT_TRUE(PointD(1.0f, 1.0f) == PointD(2.0, 2.0) / 2.0f);
    EXPECT_TRUE(PointD(1.0, 1.0) == PointD(2.0, 2.0) / 2.0);
    EXPECT_TRUE(PointD(1.0, 1.0) == PointD(2.0, 2.0) / 2.0L);

    EXPECT_TRUE(PointLD(1.0, 1.0) == PointLD(2.0, 2.0) / 2);
    EXPECT_TRUE(PointLD(1.0f, 1.0f) == (PointF)(PointLD(2.0L, 2.0L) / 2.0f));
    EXPECT_TRUE(PointLD(1.0, 1.0) == (PointF)(PointLD(2.0L, 2.0L) / 2.0));
    EXPECT_TRUE(PointLD(1.0L, 1.0L) == PointLD(2.0L, 2.0L) / 2.0L);

    // Single tests are suffient for the assignment operators because
    // they are implmented in terms of the corresponding non-assigment operators.

    Point p;

    // operator+=

    p = Point(0, 0);
    p += Point(1, 0);
    EXPECT_TRUE(Point(1, 0) == p);

    // operator-=

    p = Point(0, 0);
    p -= Point(1, 0);
    EXPECT_TRUE(Point(-1, 0) == p);

    // operator*=

    p = Point(1, 3);
    p *= 2;
    EXPECT_TRUE(Point(2, 6) == p);

    // operator/=

    p = Point(1, -3);
    p /= 2;
    EXPECT_TRUE(Point(0, -1) == p);
}

TEST(Point, non_member_ops)
{
    // Single test is sufficient because operator* is implemented in terms of the member operator.
    EXPECT_TRUE(Point(2, 2) == 1.6f * Point(1, 1));
}

TEST(Point, hash)
{
    // Check that the hash function doesn't produce the same hash for swapped coordinates.
    EXPECT_NE(std::hash<Point>()(Point(0, 1)), std::hash<Point>()(Point(1, 0)));
    EXPECT_NE(std::hash<PointF>()(PointF(0.1f, 0.9f)), std::hash<PointF>()(PointF(0.9f, 0.1f)));
    EXPECT_NE(std::hash<PointD>()(PointD(0.1, 0.9)), std::hash<PointD>()(PointD(0.9, 0.1)));
    EXPECT_NE(std::hash<PointLD>()(PointLD(0.1L, 0.9L)), std::hash<PointLD>()(PointLD(0.9L, 0.1L)));

    // Check that the correct specialization is chosen without having to explicitly specify the hash function.
    {
        std::unordered_map<Point, std::string> m;
        m[Point()] = "zero";
        m[Point(1, 1)] = "oneone";
        EXPECT_EQ(m.find(Point())->second, "zero");
        EXPECT_EQ(m.find(Point(1, 1))->second, "oneone");
    }

    {
        std::unordered_map<PointF, std::string> m;
        m[PointF()] = "zero";
        m[PointF(1.0f, 1.0f)] = "oneone";
        EXPECT_EQ(m.find(PointF())->second, "zero");
        EXPECT_EQ(m.find(PointF(1.0f, 1.0f))->second, "oneone");
    }

    {
        std::unordered_map<PointD, std::string> m;
        m[PointD()] = "zero";
        m[PointD(1.0, 1.0)] = "oneone";
        EXPECT_EQ(m.find(PointD())->second, "zero");
        EXPECT_EQ(m.find(PointD(1.0, 1.0))->second, "oneone");
    }

    {
        std::unordered_map<PointLD, std::string> m;
        m[PointLD()] = "zero";
        m[PointLD(1.0L, 1.0L)] = "oneone";
        EXPECT_EQ(m.find(PointLD())->second, "zero");
        EXPECT_EQ(m.find(PointLD(1.0L, 1.0L))->second, "oneone");
    }
}
