/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

// Devious trick here to force the test to use the header that the library did *not* use. We invert
// the definitions of GLINT_GL and GLINT_GLES before including glint/Runtime.h (which includes glint/OpenGL.h).
// This causes the test's GL headers to differ from the headers that were in effect when glint was compiled.

#if defined(GLINT_GL)
#undef GLINT_GL
#define GLINT_GLES
#else
#undef GLINT_GLES
#define GLINT_GL
#endif

#include <glint/Runtime.h>
#include <glint/GlintExceptions.h>

#include <gtest/gtest.h>

using namespace std;
using namespace glint;

// Test that, if the API client's source was compiled with gl.h (OpenGL), but the library was compiled with
// gl2.h (OpenGL ES), we throw an exception.

TEST(RuntimeMismatch, basic)
{
    try
    {
        Runtime::Ptr rt = Runtime::create();
        FAIL();
    }
    catch (const LogicException& e)
    {
#if defined(GLINT_GL)
        const char* msg = "glint::LogicException: Mismatch between glint library and calling code: glint"
                          " library was compiled with GLES2 headers, but calling code was compiled with GL headers";
#else
        const char* msg = "glint::LogicException: Mismatch between glint library and calling code: glint"
                          " library was compiled with GL headers, but calling code was compiled with GLES2 headers";
#endif
        EXPECT_EQ(msg, e.to_string());
    }
}
