/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/Shader.h>
#include <glint/Runtime.h>
#include <glint/GlintExceptions.h>

#include <gtest/gtest.h>

#include <fstream>

using namespace std;
using namespace glint;

TEST(Shader, basic)
{
    Runtime::UPtr rt = Runtime::create();
    rt->create_display(200, 200);

    cout << "Shader compilation is ";
    if (!Shader::supports_compiled_shaders())
    {
        cout << "NOT ";
    }
    cout << "supported" << endl;
}

TEST(Shader, compile)
{
    Runtime::UPtr rt = Runtime::create();
    rt->create_display(200, 200);
    ASSERT_TRUE(Shader::supports_compiled_shaders());

    const char* source = "#version 100\n"
                         "void main(void) {}";

    const char* filename = "good_shader.glsl";

    try
    {
        VertexShader::UPtr v = VertexShader::create(source);
        EXPECT_NE(0, v->id());

        ::unlink(filename);
        ofstream outfile(filename);
        outfile << source;
        outfile.close();

        VertexShader::UPtr v2 = VertexShader::create_from_file(filename);
        EXPECT_NE(0, v->id());

        FragmentShader::UPtr f = FragmentShader::create(source);
        EXPECT_NE(0, f->id());

        FragmentShader::UPtr f2 = FragmentShader::create_from_file(filename);
        EXPECT_NE(0, f->id());

        EXPECT_TRUE(Shader::supports_compiled_shaders());
    }
    catch (const LogicException& e)
    {
        EXPECT_FALSE(Shader::supports_compiled_shaders());
        EXPECT_EQ("shader compilation is not supported by this implementation", e.to_string());
    }
}

TEST(Shader, compile_error)
{
    Runtime::UPtr rt = Runtime::create();
    rt->create_display(200, 200);
    ASSERT_TRUE(Shader::supports_compiled_shaders());

    if (!Shader::supports_compiled_shaders())
    {
        return;
    }

    const char* source = "syntax error\n";
    const char* filename = "bad_shader.glsl";

    ::unlink(filename);
    ofstream outfile(filename);
    outfile << source;
    outfile.close();

    try
    {
        VertexShader::UPtr v = VertexShader::create(source);
        FAIL();
    }
    catch (const LogicException& e)
    {
        // We just print the string because the exact message is platform-specific.
        cout << "The following LogicException is expected and does not indicate test failure:" << endl;
        cout << e.to_string() << endl;
    }

    try
    {
        VertexShader::UPtr v = VertexShader::create_from_file(filename);
        FAIL();
    }
    catch (const LogicException& e)
    {
        // We just print the string because the exact message is platform-specific.
        cout << "The following LogicException is expected and does not indicate test failure:" << endl;
        cout << e.to_string() << endl;
    }

    try
    {
        FragmentShader::UPtr f = FragmentShader::create(source);
        FAIL();
    }
    catch (const LogicException& e)
    {
        // We just print the string because the exact message is platform-specific.
        cout << "The following LogicException is expected and does not indicate test failure:" << endl;
        cout << e.to_string() << endl;
    }

    try
    {
        FragmentShader::UPtr f = FragmentShader::create_from_file(filename);
        FAIL();
    }
    catch (const LogicException& e)
    {
        // We just print the string because the exact message is platform-specific.
        cout << "The following LogicException is expected and does not indicate test failure:" << endl;
        cout << e.to_string() << endl;
    }
}
