/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <gtest/gtest.h>
#include <glint/genv/egl/internal/EGLGEnv.h>
#include <glint/GraphicsDisplay.h>

using namespace std;
using namespace glint;
using namespace glint::genv::internal;
using namespace glint::genv::egl;

TEST(EGLGEnv, basic)
{
    {
        AbstractGEnv::UPtr genv = create_concrete_genv();
        cout << "Environment: " << genv->name() << " " << genv->major_version() << "." << genv->minor_version() << endl;
        cout << "Screen size: " << genv->max_display_width() << " x " << genv->max_display_height() << endl;
    } // Destructor calls shutdown()

    {
        AbstractGEnv::UPtr genv = create_concrete_genv();
        genv->shutdown();
        genv->shutdown(); // OK to call twice.
    }

    {
        AbstractGEnv::UPtr genv = create_concrete_genv();

        GraphicsDisplay* d = genv->create_display(genv->max_display_width(), genv->max_display_height(), 0, 0);
        cout << "GL version:  " << genv->gl_major_version() << "." << genv->gl_minor_version() << endl;

        EXPECT_EQ(d->width(), genv->max_display_width());
        EXPECT_EQ(d->height(), genv->max_display_height());
        EXPECT_EQ(d->xpos(), 0);
        EXPECT_EQ(d->ypos(), 0);
    }

    {
        AbstractGEnv::UPtr genv = create_concrete_genv();
        GraphicsDisplay* d = genv->create_display(200, 300, 350, 450);

        EXPECT_EQ(d->width(), 200);
        EXPECT_EQ(d->height(), 300);
        EXPECT_EQ(d->xpos(), 350);
        EXPECT_EQ(d->ypos(), 450);
    }
}
