/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <gtest/gtest.h>
#include <glint/genv/glx/GLXException.h>
#include <glint/genv/glx/internal/GLXExceptionImpl.h>

using namespace std;
using namespace glint::genv::glx;

//
// Check GLXException basic functionality (copy, assignment, accessor methods, etc.)
//

TEST(GLXException, basic)
{
    GLXException e("Hello");
    EXPECT_EQ("Hello", e.reason());
    EXPECT_STREQ("glint::genv::glx::GLXException", e.what());
    EXPECT_THROW(rethrow_exception(e.self()), GLXException);

    GLXException e2(e);
    EXPECT_EQ(e2.reason(), e.reason());
    EXPECT_STREQ(e2.what(), e.what());

    GLXException e3("blah");
    e2 = e3;
    EXPECT_EQ(e2.reason(), e3.reason());
    EXPECT_STREQ(e2.what(), e3.what());
}

// Dynamic allocataion to get around bogus function coverage reports by gcov.

TEST(GLXException, dynamic)
{
    GLXException* ep = new GLXException("Hello");
    delete ep;
    internal::GLXExceptionImpl* eip = new internal::GLXExceptionImpl("Hello");
    delete eip;
}
