/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <gtest/gtest.h>
#include <glint/genv/x11/X11Exception.h>
#include <glint/genv/x11/internal/X11ExceptionImpl.h>

using namespace std;
using namespace glint::genv::x11;

//
// Check X11Exception basic functionality (copy, assignment, accessor methods, etc.)
//

TEST(X11Exception, basic)
{
    X11Exception e("Hello");
    EXPECT_EQ("Hello", e.reason());
    EXPECT_STREQ("glint::genv::x11::X11Exception", e.what());
    EXPECT_THROW(rethrow_exception(e.self()), X11Exception);

    X11Exception e2(e);
    EXPECT_EQ(e2.reason(), e.reason());
    EXPECT_STREQ(e2.what(), e.what());

    X11Exception e3("blah");
    e2 = e3;
    EXPECT_EQ(e2.reason(), e3.reason());
    EXPECT_STREQ(e2.what(), e3.what());
}

// Dynamic allocataion to get around bogus function coverage reports by gcov.

TEST(X11Exception, dynamic)
{
    X11Exception* ep = new X11Exception("Hello");
    delete ep;
    internal::X11ExceptionImpl* eip = new internal::X11ExceptionImpl("Hello");
    delete eip;
}
