/*
 * Copyright (C) 2012 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Michi Henning <michi.henning@canonical.com>
 */

#include <glint/GlintExceptions.h>
#include <glint/util/FileIO.h>

#include <gtest/gtest.h>

#include <fstream>

using namespace std;
using namespace glint;
using namespace glint::util;

TEST(FileIO, basic)
{
    system("rm -f testfile; echo \"some chars\" >testfile");

    string s = read_text_file("testfile");
    EXPECT_EQ("some chars\n", s);

    vector<char> v = read_binary_file("testfile");
    string contents("some chars\n");
    EXPECT_EQ(vector<char>(contents.begin(), contents.end()), v);
}

TEST(FileIO, exceptions)
{
    try
    {
        read_text_file("no_such_file");
        FAIL();
    }
    catch (const FileException& e)
    {
        EXPECT_EQ("glint::FileException: cannot open \"no_such_file\": No such file or directory (errno = 2)",
                  e.to_string());
    }

    try
    {
        system("rm -f noperm; >noperm; chmod u-r noperm");
        read_text_file("noperm");
        FAIL();
    }
    catch (const FileException& e)
    {
        EXPECT_EQ("glint::FileException: cannot open \"noperm\": Permission denied (errno = 13)", e.to_string());
    }

    try
    {
        system("rm -fr testdir; mkdir testdir");
        read_text_file("testdir");
        FAIL();
    }
    catch (const FileException& e)
    {
        EXPECT_EQ("glint::FileException: \"testdir\" is not a regular file", e.to_string());
    }

    try
    {
        system("rm -fr empty; >empty");
        read_text_file("empty");
        FAIL();
    }
    catch (const FileException& e)
    {
        EXPECT_EQ("glint::FileException: file \"empty\" is empty", e.to_string());
    }
}
